@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2013  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,16 +37,7 @@ const char* parse_op_report(DB& db, const char* name)
 {
     if (name != 0 && name[0] != 0)
     {
-        const char* s;
-        int is_cod = 1;
-        for (s = name; *s && is_cod; s++)
-            if (!isdigit(*s))
-                is_cod = 0;
-
-        if (is_cod)
-            return db.rep_memo_from_cod(strtoul(name, NULL, 0)).c_str();
-        else
-            return name;
+        return name;
     } else
         return NULL;
 }
@@ -63,25 +54,26 @@ struct Importer : public Action
 
     Importer(DB& db) : db(db), import_flags(0), forced_repmemo(0) {}
 
-    virtual void operator()(const cmdline::Item& item);
+    virtual bool operator()(const cmdline::Item& item);
 };
 
-void Importer::operator()(const Item& item)
+bool Importer::operator()(const Item& item)
 {
     if (item.msgs == NULL)
     {
         fprintf(stderr, "Message #%d cannot be parsed: ignored\n", item.idx);
-        return;
+        return false;
     }
     for (size_t i = 0; i < item.msgs->size(); ++i)
     {
         Msg& msg = *(*item.msgs)[i];
         if (forced_repmemo == NULL && msg.type == MSG_GENERIC)
-            /* Put generic messages in the generic rep_cod by default */
+            /* Put generic messages in the generic report by default */
             db.import_msg(msg, NULL, import_flags);
         else
             db.import_msg(msg, forced_repmemo, import_flags);
     }
+    return true;
 }
 
 
