@@ -1,7 +1,7 @@
 /*
  * msg/defs - Common definitions
  *
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2014  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,8 +27,13 @@
 #include <cstdlib>
 #include <cstring>
 #include "dballe/core/vasprintf.h"
+#include "dballe/core/record.h"
+#include <math.h>
+#include <ostream>
+#include <iomanip>
 
 using namespace std;
+using namespace wreport;
 
 namespace dballe {
 
@@ -43,6 +48,14 @@ const char* encoding_name(Encoding enc)
 	}
 }
 
+Encoding parse_encoding(const std::string& s)
+{
+    if (s == "BUFR") return BUFR;
+    if (s == "CREX") return CREX;
+    if (s == "AOF") return AOF;
+    error_notfound::throwf("unsupported encoding '%s'", s.c_str());
+}
+
 static int str_to_val(const char* str)
 {
     if (str == NULL || str[0] == 0 || strcmp(str, "-") == 0)
@@ -61,6 +74,86 @@ Trange::Trange(const char* pind, const char* p1, const char* p2)
 {
 }
 
+bool Date::operator==(const Date& dt) const
+{
+    return year == dt.year && month == dt.month && day == dt.day;
+}
+
+bool Date::operator!=(const Date& dt) const
+{
+    return year != dt.year || month != dt.month || day != dt.day;
+}
+
+bool Date::operator<(const Date& dt) const
+{
+    if (year < dt.year) return true;
+    if (year > dt.year) return false;
+    if (month < dt.month) return true;
+    if (month > dt.month) return false;
+    return day < dt.day;
+}
+
+bool Date::operator>(const Date& dt) const
+{
+    if (year < dt.year) return false;
+    if (year > dt.year) return true;
+    if (month < dt.month) return false;
+    if (month > dt.month) return true;
+    return day > dt.day;
+}
+
+bool Time::operator==(const Time& dt) const
+{
+    return hour == dt.hour && minute == dt.minute && second == dt.second;
+}
+
+bool Time::operator!=(const Time& dt) const
+{
+    return hour != dt.hour || minute != dt.minute || second != dt.second;
+}
+
+bool Time::operator<(const Time& dt) const
+{
+    if (hour < dt.hour) return true;
+    if (hour > dt.hour) return false;
+    if (minute < dt.minute) return true;
+    if (minute > dt.minute) return false;
+    return second < dt.second;
+}
+
+bool Time::operator>(const Time& dt) const
+{
+    if (hour < dt.hour) return false;
+    if (hour > dt.hour) return true;
+    if (minute < dt.minute) return false;
+    if (minute > dt.minute) return true;
+    return second > dt.second;
+}
+
+bool Datetime::operator==(const Datetime& dt) const
+{
+    return date == dt.date && time == dt.time;
+}
+
+bool Datetime::operator!=(const Datetime& dt) const
+{
+    return date != dt.date || time != dt.time;
+}
+
+bool Datetime::operator<(const Datetime& dt) const
+{
+    if (date < dt.date) return true;
+    if (date > dt.date) return false;
+    return time < dt.time;
+}
+
+bool Datetime::operator>(const Datetime& dt) const
+{
+    if (date < dt.date) return false;
+    if (date > dt.date) return true;
+    return time > dt.time;
+}
+
 
 namespace {
 std::string fmtf( const char* f, ... )
@@ -140,7 +233,6 @@ static std::string describe_level(int ltype, int l1)
 		case 254: return "Highest top level of supercooled liquid water layer";
 		case 255: return "Missing";
 		case 256: return "Clouds";
-		case 257: return "Information about the station that generated the data";
 		case 258:
 			switch (l1) {
 				case 0: return "General cloud group";
@@ -154,7 +246,7 @@ static std::string describe_level(int ltype, int l1)
 		case 260: return fmtf("Cloud drift group %d", l1);
 		case 261: return fmtf("Cloud elevation group %d", l1);
 		case 262: return "Direction and elevation of clouds";
-	        case MISSING_INT: return "-";
+		case MISSING_INT: return "Information about the station that generated the data";
 		default:	return fmtf("%d %d", ltype, l1); break;
 	}
 }
@@ -282,12 +374,71 @@ std::string Trange::describe() const
 		case 203: return mkdesc("Vectorial maximum", p1, p2);
 		case 204: return mkdesc("Vectorial minimum", p1, p2);
 		case 205: return mkdesc("Product with a valid time ranging", p1, p2);
-		case 254: return "Instantaneous value";
+		case 254:
+			  if (p1 == 0 && p2 == 0)
+				  return "Analysis or observation, istantaneous value";
+			  else
+				  return "Forecast at t+" + format_seconds(p1) + ", instantaneous value";
 		default:  return fmtf("%d %d %d", pind, p1, p2);
 	}
 }
 
 
+int Coord::normalon(int lon)
+{
+    return ((lon + 18000000) % 36000000) - 18000000;
+}
+
+double Coord::fnormalon(double lon)
+{
+    return fmod(lon + 180.0, 360.0) - 180.0;
+}
+
+Coord::Coord(int lat, int lon)
+    : lat(lat),
+      lon(normalon(lon))
+{
+}
+
+Coord::Coord(double lat, double lon)
+    : lat(lround(lat * 100000)),
+      lon(normalon(lround(lon * 100000)))
+{
+}
+
+double Coord::dlat() const { return (double)lat/100000.0; }
+double Coord::dlon() const { return (double)lon/100000.0; }
+
+std::ostream& operator<<(std::ostream& out, const Coord& c)
+{
+    out << "(" << setprecision(5) << c.dlat()
+        << "," << setprecision(5) << c.dlon()
+        << ")";
+    return out;
+}
+
+std::ostream& operator<<(std::ostream& out, const Date& dt)
+{
+    out <<        setw(4) << setfill('0') << dt.year
+        << '-' << setw(2) << setfill('0') << (unsigned)dt.month
+        << '-' << setw(2) << setfill('0') << (unsigned)dt.day;
+    return out;
+}
+
+std::ostream& operator<<(std::ostream& out, const Time& dt)
+{
+    out <<        setw(2) << setfill('0') << (unsigned)dt.hour
+        << ':' << setw(2) << setfill('0') << (unsigned)dt.minute
+        << ':' << setw(2) << setfill('0') << (unsigned)dt.second;
+    return out;
+}
+
+std::ostream& operator<<(std::ostream& out, const Datetime& dt)
+{
+    out << dt.date << "T" << dt.time;
+    return out;
+}
+
 } // namespace dballe
 
 /* vim:set ts=4 sw=4: */
