@@ -1,7 +1,7 @@
 /*
  * dballe/record - groups of related variables
  *
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2014  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 #include "config.h"
 
+#include <sstream>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -47,6 +48,11 @@ namespace dballe {
 		assert((rec) != NULL); \
 	} while (0)
 
+std::ostream& operator<<(std::ostream& o, dba_keyword k)
+{
+    return o << Record::keyword_name(k);
+}
+
 Record::Record()
 {
 	memset(keydata, 0, sizeof(keydata));
@@ -318,6 +324,35 @@ bool Record::contains(const Record& subset) const
 	return true;
 }
 
+bool Record::contains(dba_keyword parameter) const throw()
+{
+    const Var* res = key_peek(parameter);
+    return res ? res->value() != NULL : false;
+}
+
+bool Record::contains(wreport::Varcode parameter) const throw()
+{
+    const Var* res = var_peek(parameter);
+    return res ? res->value() != NULL : false;
+}
+
+bool Record::contains_level() const throw ()
+{
+    return contains(DBA_KEY_LEVELTYPE1) || contains(DBA_KEY_L1)
+        || contains(DBA_KEY_LEVELTYPE2) || contains(DBA_KEY_L2);
+}
+
+bool Record::contains_trange() const throw ()
+{
+    return contains(DBA_KEY_PINDICATOR) || contains(DBA_KEY_P1) || contains(DBA_KEY_P2);
+}
+
+bool Record::contains_datetime() const throw ()
+{
+    return contains(DBA_KEY_YEAR) || contains(DBA_KEY_MONTH) || contains(DBA_KEY_DAY)
+        || contains(DBA_KEY_HOUR) || contains(DBA_KEY_MIN) || contains(DBA_KEY_SEC);
+}
+
 const Var* Record::key_peek(dba_keyword parameter) const throw ()
 {
 	return keydata[parameter];
@@ -474,6 +509,211 @@ const std::vector<wreport::Var*>& Record::vars() const
 	return m_vars;
 }
 
+void Record::set(const Level& lev)
+{
+    if (lev.ltype1 == MISSING_INT)
+        unset(DBA_KEY_LEVELTYPE1);
+    else
+        set(DBA_KEY_LEVELTYPE1, lev.ltype1);
+
+    if (lev.l1 == MISSING_INT)
+        unset(DBA_KEY_L1);
+    else
+        set(DBA_KEY_L1, lev.l1);
+
+    if (lev.ltype2 == MISSING_INT)
+        unset(DBA_KEY_LEVELTYPE2);
+    else
+        set(DBA_KEY_LEVELTYPE2, lev.ltype2);
+
+    if (lev.l2 == MISSING_INT)
+        unset(DBA_KEY_L2);
+    else
+        set(DBA_KEY_L2, lev.l2);
+}
+
+void Record::set(const Trange& tr)
+{
+    if (tr.pind == MISSING_INT)
+        unset(DBA_KEY_PINDICATOR);
+    else
+        set(DBA_KEY_PINDICATOR, tr.pind);
+
+    if (tr.p1 == MISSING_INT)
+        unset(DBA_KEY_P1);
+    else
+        set(DBA_KEY_P1, tr.p1);
+
+    if (tr.p2 == MISSING_INT)
+        unset(DBA_KEY_P2);
+    else
+        set(DBA_KEY_P2, tr.p2);
+}
+
+Level Record::get_level() const
+{
+    return Level(
+            get(DBA_KEY_LEVELTYPE1, MISSING_INT),
+            get(DBA_KEY_L1, MISSING_INT),
+            get(DBA_KEY_LEVELTYPE2, MISSING_INT),
+            get(DBA_KEY_L2, MISSING_INT));
+}
+
+Trange Record::get_trange() const
+{
+    return Trange(
+            get(DBA_KEY_PINDICATOR, MISSING_INT),
+            get(DBA_KEY_P1, MISSING_INT),
+            get(DBA_KEY_P2, MISSING_INT));
+}
+
+void Record::get_datetime(int (&val)[6]) const
+{
+    val[0] = get(DBA_KEY_YEAR, MISSING_INT);
+    val[1] = get(DBA_KEY_MONTH, MISSING_INT);
+    val[2] = get(DBA_KEY_DAY, MISSING_INT);
+    val[3] = get(DBA_KEY_HOUR, MISSING_INT);
+    val[4] = get(DBA_KEY_MIN, MISSING_INT);
+    val[5] = get(DBA_KEY_SEC, MISSING_INT);
+}
+
+void Record::get_datetimemin(int (&val)[6]) const
+{
+    val[0] = get(DBA_KEY_YEARMIN, MISSING_INT);
+    val[1] = get(DBA_KEY_MONTHMIN, MISSING_INT);
+    val[2] = get(DBA_KEY_DAYMIN, MISSING_INT);
+    val[3] = get(DBA_KEY_HOURMIN, MISSING_INT);
+    val[4] = get(DBA_KEY_MINUMIN, MISSING_INT);
+    val[5] = get(DBA_KEY_SECMIN, MISSING_INT);
+}
+
+void Record::get_datetimemax(int (&val)[6]) const
+{
+    val[0] = get(DBA_KEY_YEARMAX, MISSING_INT);
+    val[1] = get(DBA_KEY_MONTHMAX, MISSING_INT);
+    val[2] = get(DBA_KEY_DAYMAX, MISSING_INT);
+    val[3] = get(DBA_KEY_HOURMAX, MISSING_INT);
+    val[4] = get(DBA_KEY_MINUMAX, MISSING_INT);
+    val[5] = get(DBA_KEY_SECMAX, MISSING_INT);
+}
+
+void Record::set_datetime(const int (&val)[6])
+{
+    if (val[0] == MISSING_INT) unset(DBA_KEY_YEAR);  else set(DBA_KEY_YEAR,  val[0]);
+    if (val[1] == MISSING_INT) unset(DBA_KEY_MONTH); else set(DBA_KEY_MONTH, val[1]);
+    if (val[2] == MISSING_INT) unset(DBA_KEY_DAY);   else set(DBA_KEY_DAY,   val[2]);
+    if (val[3] == MISSING_INT) unset(DBA_KEY_HOUR);  else set(DBA_KEY_HOUR,  val[3]);
+    if (val[4] == MISSING_INT) unset(DBA_KEY_MIN);   else set(DBA_KEY_MIN,   val[4]);
+    if (val[5] == MISSING_INT) unset(DBA_KEY_SEC);   else set(DBA_KEY_SEC,   val[5]);
+}
+
+void Record::set_datetimemin(const int (&val)[6])
+{
+    if (val[0] == MISSING_INT) unset(DBA_KEY_YEARMIN);  else set(DBA_KEY_YEARMIN,  val[0]);
+    if (val[1] == MISSING_INT) unset(DBA_KEY_MONTHMIN); else set(DBA_KEY_MONTHMIN, val[1]);
+    if (val[2] == MISSING_INT) unset(DBA_KEY_DAYMIN);   else set(DBA_KEY_DAYMIN,   val[2]);
+    if (val[3] == MISSING_INT) unset(DBA_KEY_HOURMIN);  else set(DBA_KEY_HOURMIN,  val[3]);
+    if (val[4] == MISSING_INT) unset(DBA_KEY_MINUMIN);  else set(DBA_KEY_MINUMIN,  val[4]);
+    if (val[5] == MISSING_INT) unset(DBA_KEY_SECMIN);   else set(DBA_KEY_SECMIN,   val[5]);
+}
+
+void Record::set_datetimemax(const int (&val)[6])
+{
+    if (val[0] == MISSING_INT) unset(DBA_KEY_YEARMAX);  else set(DBA_KEY_YEARMAX,  val[0]);
+    if (val[1] == MISSING_INT) unset(DBA_KEY_MONTHMAX); else set(DBA_KEY_MONTHMAX, val[1]);
+    if (val[2] == MISSING_INT) unset(DBA_KEY_DAYMAX);   else set(DBA_KEY_DAYMAX,   val[2]);
+    if (val[3] == MISSING_INT) unset(DBA_KEY_HOURMAX);  else set(DBA_KEY_HOURMAX,  val[3]);
+    if (val[4] == MISSING_INT) unset(DBA_KEY_MINUMAX);  else set(DBA_KEY_MINUMAX,  val[4]);
+    if (val[5] == MISSING_INT) unset(DBA_KEY_SECMAX);   else set(DBA_KEY_SECMAX,   val[5]);
+}
+
+Datetime Record::get_datetime() const
+{
+    return Datetime(
+        get(DBA_KEY_YEAR).enqi(),
+        get(DBA_KEY_MONTH, 1),
+        get(DBA_KEY_DAY, 1),
+        get(DBA_KEY_HOUR, 0),
+        get(DBA_KEY_MIN, 0),
+        get(DBA_KEY_SEC, 0));
+}
+
+void Record::unset_datetime()
+{
+    unset(DBA_KEY_YEAR);
+    unset(DBA_KEY_MONTH);
+    unset(DBA_KEY_DAY);
+    unset(DBA_KEY_HOUR);
+    unset(DBA_KEY_MIN);
+    unset(DBA_KEY_SEC);
+}
+
+void Record::unset_datetimemin()
+{
+    unset(DBA_KEY_YEARMIN);
+    unset(DBA_KEY_MONTHMIN);
+    unset(DBA_KEY_DAYMIN);
+    unset(DBA_KEY_HOURMIN);
+    unset(DBA_KEY_MINUMIN);
+    unset(DBA_KEY_SECMIN);
+}
+
+void Record::unset_datetimemax()
+{
+    unset(DBA_KEY_YEARMAX);
+    unset(DBA_KEY_MONTHMAX);
+    unset(DBA_KEY_DAYMAX);
+    unset(DBA_KEY_HOURMAX);
+    unset(DBA_KEY_MINUMAX);
+    unset(DBA_KEY_SECMAX);
+}
+
+void Record::set(const Datetime& dt)
+{
+    set(DBA_KEY_YEAR,  (int)dt.date.year);
+    set(DBA_KEY_MONTH, (int)dt.date.month);
+    set(DBA_KEY_DAY,   (int)dt.date.day);
+    set(DBA_KEY_HOUR,  (int)dt.time.hour);
+    set(DBA_KEY_MIN,   (int)dt.time.minute);
+    set(DBA_KEY_SEC,   (int)dt.time.second);
+}
+
+void Record::setmin(const Datetime& dt)
+{
+    set(DBA_KEY_YEARMIN,  (int)dt.date.year);
+    set(DBA_KEY_MONTHMIN, (int)dt.date.month);
+    set(DBA_KEY_DAYMIN,   (int)dt.date.day);
+    set(DBA_KEY_HOURMIN,  (int)dt.time.hour);
+    set(DBA_KEY_MINUMIN,  (int)dt.time.minute);
+    set(DBA_KEY_SECMIN,   (int)dt.time.second);
+}
+
+void Record::setmax(const Datetime& dt)
+{
+    set(DBA_KEY_YEARMAX,  (int)dt.date.year);
+    set(DBA_KEY_MONTHMAX, (int)dt.date.month);
+    set(DBA_KEY_DAYMAX,   (int)dt.date.day);
+    set(DBA_KEY_HOURMAX,  (int)dt.time.hour);
+    set(DBA_KEY_MINUMAX,  (int)dt.time.minute);
+    set(DBA_KEY_SECMAX,   (int)dt.time.second);
+}
+
+void Record::set_datetime(int ye, int mo, int da, int ho, int mi, int se)
+{
+    set(DBA_KEY_YEAR,  ye);
+    set(DBA_KEY_MONTH, mo);
+    set(DBA_KEY_DAY,   da);
+    set(DBA_KEY_HOUR,  ho);
+    set(DBA_KEY_MIN,   mi);
+    set(DBA_KEY_SEC,   se);
+}
+
+void Record::set(const Coord& c)
+{
+    set(DBA_KEY_LAT, c.lat);
+    set(DBA_KEY_LON, c.lon);
+}
+
 void Record::set_ana_context()
 {
 	key(DBA_KEY_YEAR).seti(1000);
@@ -482,54 +722,66 @@ void Record::set_ana_context()
 	key(DBA_KEY_HOUR).seti(0);
 	key(DBA_KEY_MIN).seti(0);
 	key(DBA_KEY_SEC).seti(0);
-	key(DBA_KEY_LEVELTYPE1).seti(257);
+	unset(DBA_KEY_LEVELTYPE1);
 	unset(DBA_KEY_L1);
 	unset(DBA_KEY_LEVELTYPE2);
 	unset(DBA_KEY_L2);
 	unset(DBA_KEY_PINDICATOR);
 	unset(DBA_KEY_P1);
 	unset(DBA_KEY_P2);
-	/* DBA_RUN_OR_RETURN(dba_record_key_seti(rec, DBA_KEY_REP_COD, 254)); */
+}
+
+bool Record::is_ana_context() const
+{
+    return get(DBA_KEY_YEAR, MISSING_INT) == 1000;
 }
 
 void Record::set_from_string(const char* str)
 {
-	/* Split the input as name=val */
-	const char* s;
-	const char* val;
-	Varcode varcode;
-	
-	if ((s = strchr(str, '=')) == NULL)
-		error_consistency::throwf("there should be an = between the name and the value in '%s'", str);
+    // Split the input as name=val
+    const char* s = strchr(str, '=');
 
-//		name = strndup(queryparm, s - queryparm);
-	val = s + 1;
+    if (!s) error_consistency::throwf("there should be an = between the name and the value in '%s'", str);
 
-	/* First see if it's an alias or a variable */
-	if ((varcode = varcode_alias_resolve_substring(str, s - str)) != 0 || str[0] == 'B')
-	{
-		if (varcode == 0)
-			varcode = WR_STRING_TO_VAR(str + 1);
+    string key(str, s - str);
+    set_from_string(key.c_str(), s + 1);
+}
 
-		/* Query informations about the parameter */
-		Varinfo info = varinfo(varcode);
-		if (info->is_string())
-			var(varcode).setc(val);
-		else
-			var(varcode).setd(strtod(val, 0));
-	} else {
-		/* Else handle a normal keyword */
-		dba_keyword param = keyword_byname_len(str, s - str);
-		if (param == DBA_KEY_ERROR)
-			error_notfound::throwf("keyword \"%.*s\" does not exist", (int)(s - str), str);
+void Record::set_from_string(const char* name, const char* val)
+{
+    dba_keyword k = Record::keyword_byname(name);
 
-		/* Query informations about the parameter */
-		Varinfo info = keyword_info(param);
-		if (info->is_string())
-			key(param).setc(val);
-		else
-			key(param).setd(strtod(val, 0));
-	}
+    if (k != DBA_KEY_ERROR)
+        key(k).set_from_formatted(val);
+    else
+        var(resolve_varcode_safe(name)).set_from_formatted(val);
+}
+
+std::string Record::to_string() const
+{
+    std::stringstream s;
+    bool first = true;
+
+    for (int i = 0; i < KEYWORD_TABLE_SIZE; ++i)
+        if (keydata[i] != NULL)
+        {
+            if (first)
+                first = false;
+            else
+                s << ",";
+            s << keyword_name((dba_keyword)i) << "=" << keydata[i]->format("");
+        }
+
+    for (vector<Var*>::const_iterator i = m_vars.begin(); i != m_vars.end(); ++i)
+    {
+        if (first)
+            first = false;
+        else
+            s << ",";
+        s << wreport::varcode_format((*i)->code()) << "=" << (*i)->format("");
+    }
+
+    return s.str();
 }
 
 void Record::print(FILE* out) const
@@ -595,24 +847,24 @@ void dba_record_diff(dba_record rec1, dba_record rec2, int* diffs, FILE* out)
 
 static inline int peek_int(const Record& rec, dba_keyword key)
 {
-	const char* s = rec.key_peek_value(key);
-	return s != NULL ? strtol(s, 0, 10) : -1;
+    const char* s = rec.key_peek_value(key);
+    return s != NULL ? strtol(s, 0, 10) : MISSING_INT;
 }
 
 static inline int min_with_undef(int v1, int v2)
 {
-	if (v1 == -1)
-		return v2;
-	if (v2 == -1)
-		return v1;
-	return v1 < v2 ? v1 : v2;
+    if (v1 == MISSING_INT)
+        return v2;
+    if (v2 == MISSING_INT)
+        return v1;
+    return v1 < v2 ? v1 : v2;
 }
 
 static inline int max_with_undef(int v1, int v2)
 {
-	if (v1 == -1)
+	if (v1 == MISSING_INT)
 		return v2;
-	if (v2 == -1)
+	if (v2 == MISSING_INT)
 		return v1;
 	return v1 > v2 ? v1 : v2;
 }
@@ -621,7 +873,7 @@ static inline int max_days(int y, int m)
 {
 	int days[] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
 	if (m != 2)
-		return days[m-1];
+		return days[m - 1];
 	else
 		return (y % 4 == 0 && (y % 100 != 0 || y % 400 == 0)) ? 29 : 28;
 }
@@ -645,9 +897,9 @@ void Record::parse_date_extremes(int* minvalues, int* maxvalues) const
 		minvalues[i] = max_with_undef(val, min);
 		maxvalues[i] = min_with_undef(val, max);
 
-		if (i > 0 &&
-		  ((minvalues[i-1] == -1 && minvalues[i] != -1) ||
-		   (maxvalues[i-1] == -1 && maxvalues[i] != -1)))
+        if (i > 0 &&
+           ((minvalues[i - 1] == MISSING_INT && minvalues[i] != MISSING_INT) ||
+            (maxvalues[i - 1] == MISSING_INT && maxvalues[i] != MISSING_INT)))
 		{
 			Varinfo key1 = keyword_info(names[i - 1]);
 			Varinfo key2 = keyword_info(names[i]);
@@ -657,27 +909,27 @@ void Record::parse_date_extremes(int* minvalues, int* maxvalues) const
 		}
 	}
 
-	/* Now values is either 6 times -1, 6 values, or X values followed by 6-X times -1 */
+	/* Now values is either 6 times MISSING_INT, 6 values, or X values followed by 6-X times MISSING_INT */
 
 	/* If one of the extremes has been selected, fill in the blanks */
 
-	if (minvalues[0] != -1)
-	{
-		minvalues[1] = minvalues[1] != -1 ? minvalues[1] : 1;
-		minvalues[2] = minvalues[2] != -1 ? minvalues[2] : 1;
-		minvalues[3] = minvalues[3] != -1 ? minvalues[3] : 0;
-		minvalues[4] = minvalues[4] != -1 ? minvalues[4] : 0;
-		minvalues[5] = minvalues[5] != -1 ? minvalues[5] : 0;
-	}
+    if (minvalues[0] != MISSING_INT)
+    {
+        minvalues[1] = minvalues[1] != MISSING_INT ? minvalues[1] : 1;
+        minvalues[2] = minvalues[2] != MISSING_INT ? minvalues[2] : 1;
+        minvalues[3] = minvalues[3] != MISSING_INT ? minvalues[3] : 0;
+        minvalues[4] = minvalues[4] != MISSING_INT ? minvalues[4] : 0;
+        minvalues[5] = minvalues[5] != MISSING_INT ? minvalues[5] : 0;
+    }
 
-	if (maxvalues[0] != -1)
-	{
-		maxvalues[1] = maxvalues[1] != -1 ? maxvalues[1] : 12;
-		maxvalues[2] = maxvalues[2] != -1 ? maxvalues[2] : max_days(maxvalues[0], maxvalues[1]);
-		maxvalues[3] = maxvalues[3] != -1 ? maxvalues[3] : 23;
-		maxvalues[4] = maxvalues[4] != -1 ? maxvalues[4] : 59;
-		maxvalues[5] = maxvalues[5] != -1 ? maxvalues[5] : 59;
-	}
+    if (maxvalues[0] != MISSING_INT)
+    {
+        maxvalues[1] = maxvalues[1] != MISSING_INT ? maxvalues[1] : 12;
+        maxvalues[2] = maxvalues[2] != MISSING_INT ? maxvalues[2] : max_days(maxvalues[0], maxvalues[1]);
+        maxvalues[3] = maxvalues[3] != MISSING_INT ? maxvalues[3] : 23;
+        maxvalues[4] = maxvalues[4] != MISSING_INT ? maxvalues[4] : 59;
+        maxvalues[5] = maxvalues[5] != MISSING_INT ? maxvalues[5] : 59;
+    }
 }
 
 void Record::parse_date(int* values) const
@@ -687,7 +939,7 @@ void Record::parse_date(int* values) const
     {
         values[i] = peek_int(*this, names[i]);
 
-        if (i > 0 && (values[i-1] == -1 && values[i] != -1))
+        if (i > 0 && (values[i-1] == MISSING_INT && values[i] != MISSING_INT))
         {
             Varinfo key1 = keyword_info(names[i - 1]);
             Varinfo key2 = keyword_info(names[i]);
@@ -697,17 +949,17 @@ void Record::parse_date(int* values) const
         }
     }
 
-    /* Now values is either 6 times -1, 6 values, or X values followed by 6-X times -1 */
+    /* Now values is either 6 times MISSING_INT, 6 values, or X values followed by 6-X times MISSING_INT */
 
     /* If one of the extremes has been selected, fill in the blanks */
 
-    if (values[0] != -1)
+    if (values[0] != MISSING_INT)
     {
-        values[1] = values[1] != -1 ? values[1] : 1;
-        values[2] = values[2] != -1 ? values[2] : 1;
-        values[3] = values[3] != -1 ? values[3] : 0;
-        values[4] = values[4] != -1 ? values[4] : 0;
-        values[5] = values[5] != -1 ? values[5] : 0;
+        values[1] = values[1] != MISSING_INT ? values[1] : 1;
+        values[2] = values[2] != MISSING_INT ? values[2] : 1;
+        values[3] = values[3] != MISSING_INT ? values[3] : 0;
+        values[4] = values[4] != MISSING_INT ? values[4] : 0;
+        values[5] = values[5] != MISSING_INT ? values[5] : 0;
     }
 }
 
@@ -762,7 +1014,7 @@ matcher::Result MatchedRecord::match_date(const int* min, const int* max) const
 {
     int date[6];
     r.parse_date(date);
-    if (date[0] == -1) return matcher::MATCH_NA;
+    if (date[0] == MISSING_INT) return matcher::MATCH_NA;
     return Matched::date_in_range(date, min, max);
 }
 
