@@ -1,4 +1,4 @@
-/* C++ code produced by gperf version 3.0.3 */
+/* C++ code produced by gperf version 3.0.4 */
 /* Command-line: gperf  */
 /* Computed positions: -k'1-2,$' */
 
@@ -40,7 +40,7 @@ using namespace wreport;
 namespace dballe {
 struct tagdef { const char* tag; dba_keyword index; };
 
-#define TOTAL_KEYWORDS 48
+#define TOTAL_KEYWORDS 47
 #define MIN_WORD_LENGTH 2
 #define MAX_WORD_LENGTH 11
 #define MIN_HASH_VALUE 6
@@ -126,7 +126,7 @@ static struct tagdef wordlist[] =
     {""},
     {"pindicator",  DBA_KEY_PINDICATOR},
     {"latmin",      DBA_KEY_LATMIN},
-    {"rep_cod",     DBA_KEY_REP_COD},
+    {""},
     {"day",         DBA_KEY_DAY},
     {""},
     {"month",       DBA_KEY_MONTH},
@@ -193,7 +193,6 @@ struct keyword_infos
 		infos[DBA_KEY_PRIORITY	 ].set(0, "Priority of this type of report",         "NUMBER", DEF_MAXINT);
 		infos[DBA_KEY_PRIOMAX	 ].set(0, "Maximum priority of reports queried",     "NUMBER", DEF_MAXINT);
 		infos[DBA_KEY_PRIOMIN	 ].set(0, "Minimum priority of reports queried",     "NUMBER", DEF_MAXINT);
-		infos[DBA_KEY_REP_COD	 ].set(0, "Type of report",                          "NUMBER", DEF_MAXINT);
 		infos[DBA_KEY_REP_MEMO	 ].set(0, "Mnemonic alias for type of report",     "CCITTIA5", DEF_STRING(20));
 		infos[DBA_KEY_ANA_ID	 ].set(0, "Station database ID",                    "NUMERIC", DEF_MAXINT);
 		infos[DBA_KEY_MOBILE	 ].set(0, "Station is mobile",                      "NUMERIC", DEF_MAXINT);
@@ -253,7 +252,6 @@ static const char* keyword_names[] = {
 	"priority",
 	"priomax",
 	"priomin",
-	"rep_cod",
 	"rep_memo",
 	"ana_id",
 	"mobile",
