@@ -1,7 +1,7 @@
 /*
- * DB-ALLe - Archive for punctual meteorological data
+ * core/test-utils-core - Test utility functions for the core module
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,77 +19,21 @@
  * Author: Enrico Zini <enrico@enricozini.com>
  */
 
-#include "test-utils-core.h"
-#include "dballe/core/file_internals.h"
-
+#include "dballe/core/test-utils-core.h"
+#include <wibble/string.h>
 #include <unistd.h>
 #include <sys/types.h>
 #include <pwd.h>
 
-namespace tut_dballe {
+using namespace wibble;
+using namespace wreport;
+using namespace std;
+
+namespace dballe {
+namespace tests {
 
 static std::string tag;
 
-void test_tag(const std::string& ttag)
-{
-	tag = ttag;
-}
-
-void test_untag()
-{
-	tag = string();
-}
-
-DBAException::DBAException(const char* file, int line)
-{
-	std::stringstream ss;
-	ss << file << ":" << line << ": ";
-	if (!tag.empty())
-		ss << "[" << tag << "] ";
-	ss << "Error " << dba_error_get_code();
-	ss << " (" << dba_error_get_message() << ") while " << dba_error_get_context();
-
-	const char* details = dba_error_get_details();
-	if (details == NULL)
-		ss << endl;
-	else
-		ss << ".  Details:" << endl << details << endl;
-
-	m_what = ss.str();
-}
-
-std::string __ensure_errmsg(std::string file, int line, std::string msg)
-{
-	std::stringstream ss;
-	ss << file << ":" << line << ": ";
-	if (!tag.empty())
-		ss << "[" << tag << "] ";
-	ss << "'" << msg << "'";
-	return ss.str();
-}
-
-void _ensure_var_undef(const char* file, int line, dba_var var)
-{
-	inner_ensure_equals(dba_var_value(var), (const char*)0);
-}
-void _ensure_var_equals(const char* file, int line, dba_var var, int val)
-{
-	int v;
-	INNER_CHECKED(dba_var_enqi(var, &v));
-	inner_ensure_equals(v, val);
-}
-void _ensure_var_equals(const char* file, int line, dba_var var, double val)
-{
-	double v;
-	INNER_CHECKED(dba_var_enqd(var, &v));
-	inner_ensure_equals(v, val);
-}
-void _ensure_var_equals(const char* file, int line, dba_var var, const string& val)
-{
-	const char* v;
-	INNER_CHECKED(dba_var_enqc(var, &v));
-	inner_ensure_equals(string(v), val);
-}
 
 /*
 static void _ensureRecordHas(const char* file, int line, dba_record rec, const char* key, int val)
@@ -113,47 +57,48 @@ static void _ensureRecordHas(const char* file, int line, dba_record rec, const c
 #define ensureRecordHas(...) _ensureRecordHas(__FILE__, __LINE__, __VA_ARGS__)
 */
 
-const static dba_varcode generator_varcodes[] = {
-	DBA_VAR(0,  1,   1),
-	DBA_VAR(0,  1,   2),
-	DBA_VAR(0,  1,   8),
-	DBA_VAR(0,  1,  11),
-	DBA_VAR(0,  1,  12),
-	DBA_VAR(0,  1,  13),
-	DBA_VAR(0,  2,   1),
-	DBA_VAR(0,  2,   2),
-	DBA_VAR(0,  2,   5),
-	DBA_VAR(0,  2,  11),
-	DBA_VAR(0,  2,  12),
-	DBA_VAR(0,  2,  61),
-	DBA_VAR(0,  2,  62),
-	DBA_VAR(0,  2,  63),
-	DBA_VAR(0,  2,  70),
-	DBA_VAR(0,  4,   1),
-	DBA_VAR(0,  4,   2),
-	DBA_VAR(0,  4,   3),
-	DBA_VAR(0,  4,   4),
-	DBA_VAR(0,  4,   5),
-	DBA_VAR(0,  5,   1),
-	DBA_VAR(0,  6,   1),
-	DBA_VAR(0,  7,   1),
-	DBA_VAR(0,  7,   2),
-	DBA_VAR(0,  7,  31),
-	DBA_VAR(0,  8,   1),
-	DBA_VAR(0,  8,   4),
-	DBA_VAR(0,  8,  21),
-	DBA_VAR(0, 10,   3),
-	DBA_VAR(0, 10,   4),
-	DBA_VAR(0, 10,  51),
-	DBA_VAR(0, 10,  61),
-	DBA_VAR(0, 10,  63),
-	DBA_VAR(0, 10, 197),
-	DBA_VAR(0, 11,   1),
-	DBA_VAR(0, 11,   2),
-	DBA_VAR(0, 11,   3),
-	DBA_VAR(0, 11,   4),
+const static Varcode generator_varcodes[] = {
+	WR_VAR(0,  1,   1),
+	WR_VAR(0,  1,   2),
+	WR_VAR(0,  1,   8),
+	WR_VAR(0,  1,  11),
+	WR_VAR(0,  1,  12),
+	WR_VAR(0,  1,  13),
+	WR_VAR(0,  2,   1),
+	WR_VAR(0,  2,   2),
+	WR_VAR(0,  2,   5),
+	WR_VAR(0,  2,  11),
+	WR_VAR(0,  2,  12),
+	WR_VAR(0,  2,  61),
+	WR_VAR(0,  2,  62),
+	WR_VAR(0,  2,  63),
+	WR_VAR(0,  2,  70),
+	WR_VAR(0,  4,   1),
+	WR_VAR(0,  4,   2),
+	WR_VAR(0,  4,   3),
+	WR_VAR(0,  4,   4),
+	WR_VAR(0,  4,   5),
+	WR_VAR(0,  5,   1),
+	WR_VAR(0,  6,   1),
+	WR_VAR(0,  7,  30),
+	WR_VAR(0,  7,   2),
+	WR_VAR(0,  7,  31),
+	WR_VAR(0,  8,   1),
+	WR_VAR(0,  8,   4),
+	WR_VAR(0,  8,  21),
+	WR_VAR(0, 10,   3),
+	WR_VAR(0, 10,   4),
+	WR_VAR(0, 10,  51),
+	WR_VAR(0, 10,  61),
+	WR_VAR(0, 10,  63),
+	WR_VAR(0, 10, 197),
+	WR_VAR(0, 11,   1),
+	WR_VAR(0, 11,   2),
+	WR_VAR(0, 11,   3),
+	WR_VAR(0, 11,   4),
 };
 
+#if 0
 generator::~generator()
 {
 	for (std::vector<dba_record>::iterator i = reused_pseudoana_fixed.begin();
@@ -238,6 +183,9 @@ dba_err generator::fill_record(dba_record rec)
 	return dba_error_ok();
 }
 
+#endif
+
+#if 0
 dba_err read_file(dba_encoding type, const std::string& name, dba_raw_consumer& cons)
 {
 	dba_err err = DBA_OK;
@@ -259,91 +207,120 @@ cleanup:
 	if (raw) dba_rawmsg_delete(raw);
 	return err == DBA_OK ? dba_error_ok() : err;
 }
+#endif
 
-dba_file _open_test_data(const char* file, int line, const char* filename, dba_encoding type)
+std::string datafile(const std::string& fname)
 {
+    // Skip appending the test data path for pathnames starting with ./
+    if (fname[0] == '.') return fname;
 	const char* testdatadirenv = getenv("DBA_TESTDATA");
 	std::string testdatadir = testdatadirenv ? testdatadirenv : ".";
-	dba_file res;
-
-	INNER_CHECKED(dba_file_create(type, (testdatadir + "/" + filename).c_str(), "r", &res));
-
-	return res;
+	return testdatadir + "/" + fname;
 }
 
-dba_rawmsg _read_rawmsg(const char* file, int line, const char* filename, dba_encoding type)
+auto_ptr<File> _open_test_data(const wibble::tests::Location& loc, const char* filename, Encoding type)
 {
-	dba_file input = _open_test_data(file, line, filename, type);
-	dba_rawmsg rawmsg;
-	int found;
-
-	// Read the sample message
-	INNER_CHECKED(dba_rawmsg_create(&rawmsg));
-	INNER_CHECKED(dba_file_read(input, rawmsg, &found));
-	inner_ensure_equals(found, 1);
-
-	rawmsg->file = NULL;
-	dba_file_delete(input);
-
-	return rawmsg;
-}
-
-static dba_err slurp_file_read(dba_file file, dba_rawmsg msg, int* found)
-{
-	FILE* in = file->fd;
-
-	/* Reset bufr_message data in case this message has been used before */
-	dba_rawmsg_reset(msg);
-	msg->offset = ftell(in);
-
-	/* Read the entire file contents */
-	while (!feof(in))
-	{
-		unsigned char c;
-		if (fread(&c, 1, 1, in) == 1)
-		{
-			if (msg->len >= msg->alloclen)
-				DBA_RUN_OR_RETURN(dba_rawmsg_expand_buffer(msg));
-			msg->buf[msg->len++] = c;
-		}
+	try {
+		return auto_ptr<File>(File::create(type, datafile(filename), "r"));
+	} catch (wreport::error& e) {
+		throw tut::failure(loc.msg(e.what()));
 	}
-	
-	msg->encoding = BUFR;
-	msg->file = file;
-	*found = msg->len != 0;
-	return dba_error_ok();
-}
-static void slurp_file_delete(dba_file file)
-{
-	free(file);
-}
-static dba_err slurp_file_create(dba_encoding type, FILE* fd, const char* mode, dba_file* file)
-{
-	*file = (dba_file)calloc(1, sizeof(struct _dba_file));
-	if (*file == NULL)
-		return dba_error_alloc("allocating new _dba_file");
-	(*file)->fun_delete = slurp_file_delete;
-	(*file)->fun_read = slurp_file_read;
-	(*file)->fun_write = dba_file_default_write_impl;
-	return dba_error_ok();
 }
 
-DbaFileSlurpOnly::DbaFileSlurpOnly()
+auto_ptr<Rawmsg> _read_rawmsg(const wibble::tests::Location& loc, const char* filename, Encoding type)
 {
-	oldBufr = (void*)dba_file_aof_create;
-	oldCrex = (void*)dba_file_bufr_create;
-	oldAof = (void*)dba_file_crex_create;
-	dba_file_aof_create = slurp_file_create;
-	dba_file_bufr_create = slurp_file_create;
-	dba_file_crex_create = slurp_file_create;
-}
-DbaFileSlurpOnly::~DbaFileSlurpOnly()
-{
-	dba_file_aof_create = (dba_file_create_fun)oldBufr;
-	dba_file_bufr_create = (dba_file_create_fun)oldCrex;
-	dba_file_crex_create = (dba_file_create_fun)oldAof;
+	try {
+		auto_ptr<File> f = _open_test_data(loc, filename, type);
+		auto_ptr<Rawmsg> res(new Rawmsg);
+
+		inner_ensure(f->read(*res));
+
+		return res;
+	} catch (wreport::error& e) {
+		throw tut::failure(loc.msg(e.what()));
+	}
 }
 
+static std::string format_name(dba_keyword k) { return Record::keyword_name(k); }
+static std::string format_name(wreport::Varcode c) { return format_code(c); }
+static std::string format_name(const char* c) { return c; }
+
+template<typename K>
+void TestRecordValEqual<K>::check(WIBBLE_TEST_LOCPRM) const
+{
+    const wreport::Var* evar = expected.peek(name);
+    const wreport::Var* avar = actual.peek(name);
+
+    if (with_missing_int && evar && evar->enq(MISSING_INT) == MISSING_INT)
+        evar = NULL;
+    if (with_missing_int && avar && avar->enq(MISSING_INT) == MISSING_INT)
+        avar = NULL;
+
+    if (!evar && !avar) return;
+    if (!evar || !avar || evar->code() != avar->code() || !evar->value_equals(*avar))
+    {
+        std::stringstream ss;
+        ss << "records differ on " << format_name(name) << ": ";
+        if (!evar)
+            ss << "it is expected unset";
+        else
+            ss << evar->format() << " is expected";
+        ss << ", but actual ";
+        if (!avar)
+            ss << "is unset";
+        else
+            ss << "has " << avar->format();
+        wibble_test_location.fail_test(ss.str());
+    }
+}
+template class TestRecordValEqual<dba_keyword>;
+template class TestRecordValEqual<wreport::Varcode>;
+template class TestRecordValEqual<const char*>;
+
+void TestRecordVarsEqual::check(WIBBLE_TEST_LOCPRM) const
+{
+    WIBBLE_TEST_INFO(locinfo);
+    locinfo() << "Expected: " << expected.to_string() << " actual: " << actual.to_string();
+
+    const vector<Var*>& vars1 = actual.vars();
+    const vector<Var*>& vars2 = expected.vars();
+
+    if (vars1.size() != vars2.size())
+    {
+        std::stringstream ss;
+        ss << "records have a different number of variables. Expected is " << vars2.size() << " and actual has " << vars1.size();
+        wibble_test_location.fail_test(ss.str());
+    }
+
+    for (unsigned i = 0; i < vars1.size(); ++i)
+    {
+        if (*vars1[i] != *vars2[i])
+        {
+            std::stringstream ss;
+            ss << "records variables differ at position " << i << ": expected is "
+               << varcode_format(vars2[i]->code()) << "=" << vars2[i]->format("")
+               << " and actual has "
+               << varcode_format(vars1[i]->code()) << "=" << vars1[i]->format("");
+            wibble_test_location.fail_test(ss.str());
+        }
+    }
+}
+
+void set_record_from_string(Record& rec, const std::string& s)
+{
+    str::Split splitter(", ", s);
+    for (str::Split::const_iterator i = splitter.begin(); i != splitter.end(); ++i)
+        rec.set_from_string(i->c_str());
+}
+
+Record record_from_string(const std::string& s)
+{
+    Record res;
+    set_record_from_string(res, s);
+    return res;
+}
+
+}
 }
 
 // vim:set ts=4 sw=4:
