@@ -31,6 +31,7 @@
 #include <wreport/var.h>
 #include <memory>
 #include <string>
+#include <set>
 
 namespace dballe {
 
@@ -41,6 +42,19 @@ namespace dballe {
 /// Return a Varinfo entry from the local B table
 wreport::Varinfo varinfo(wreport::Varcode code);
 
+/**
+ * Resolve a varcode name to a varcode proper.
+ */
+wreport::Varcode resolve_varcode(const char* name);
+
+/// Resolve a varcode name to a varcode proper, performing careful validation
+wreport::Varcode resolve_varcode_safe(const char* name);
+
+/// Resolve a varcode name to a varcode proper, performing careful validation
+wreport::Varcode resolve_varcode_safe(const std::string& name);
+
+/// Resolve a comma-separated varcode list performing careful validation, inserting results in \a out
+void resolve_varlist_safe(const std::string& varlist, std::set<wreport::Varcode>& out);
 
 /// Create a new Var, from the local B table, with undefined value
 static inline wreport::Var var(wreport::Varcode code) { return wreport::Var(varinfo(code)); }
