@@ -1,7 +1,7 @@
 /*
  * db/internals - Internal support infrastructure for the DB
  *
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2013  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,11 +40,15 @@
  */
 #define DBA_USE_TRANSACTIONS
 
+// Bit values for server/driver quirks
+#define DBA_DB_QUIRK_NO_ROWCOUNT_IN_DIAG (1 << 0)
+
 /* Define this to enable referential integrity */
 #undef USE_REF_INT
 
 namespace dballe {
 namespace db {
+struct Statement;
 
 /** Trace macros internally used for debugging
  * @{
@@ -125,13 +129,25 @@ struct Connection
     bool connected;
     /** Type of SQL server we are connected to */
     enum ServerType server_type;
+    /// Bitfield of quirks we should be aware of when using ODBC
+    unsigned server_quirks;
 
+protected:
+    /** Precompiled LAST_INSERT_ID (or equivalent) SQL statement */
+    db::Statement* stm_last_insert_id;
+    /** ID of the last autogenerated primary key */
+    DBALLE_SQL_C_SINT_TYPE m_last_insert_id;
+
+public:
     Connection();
     ~Connection();
 
     void connect(const char* dsn, const char* user, const char* password);
+    void connect_file(const std::string& fname);
     void driver_connect(const char* config);
     std::string driver_name();
+    std::string driver_version();
+    void get_info(SQLUSMALLINT info_type, SQLINTEGER& res);
     void set_autocommit(bool val);
 
     /// Commit a transaction
@@ -140,6 +156,44 @@ struct Connection
     /// Rollback a transaction
     void rollback();
 
+    /// Check if the database contains a table
+    bool has_table(const std::string& name);
+
+    /**
+     * Get a value from the settings table.
+     *
+     * Returns the empty string if the table does not exist.
+     */
+    std::string get_setting(const std::string& key);
+
+    /**
+     * Set a value in the settings table.
+     *
+     * The table is created if it does not exist.
+     */
+    void set_setting(const std::string& key, const std::string& value);
+
+    /// Drop the settings table
+    void drop_settings();
+
+	/**
+	 * Delete a table in the database if it exists, otherwise do nothing.
+	 */
+	void drop_table_if_exists(const char* name);
+
+	/**
+	 * Delete a sequence in the database if it exists, otherwise do nothing.
+	 */
+	void drop_sequence_if_exists(const char* name);
+
+    /**
+     * Return LAST_INSERT_ID or LAST_INSER_ROWID or whatever is appropriate for
+     * the current database, if supported.
+     *
+     * If not supported, an exception is thrown.
+     */
+    int get_last_insert_id();
+
 protected:
     void init_after_connect();
 
@@ -165,7 +219,7 @@ struct Transaction
 /// ODBC statement
 struct Statement
 {
-    //Connection& conn;
+    const Connection& conn;
     SQLHSTMT stm;
     /// If non-NULL, ignore all errors with this code
     const char* ignore_error;
@@ -195,6 +249,7 @@ struct Statement
     void bind_out(int idx, char* val, SQLLEN buflen);
     void bind_out(int idx, char* val, SQLLEN buflen, SQLLEN& ind);
     void bind_out(int idx, SQL_TIMESTAMP_STRUCT& val);
+    void bind_out(int idx, SQL_TIMESTAMP_STRUCT& val, SQLLEN& ind);
 
     void prepare(const char* query);
     void prepare(const char* query, int qlen);
@@ -221,9 +276,14 @@ struct Statement
     bool fetch();
     bool fetch_expecting_one();
     void close_cursor();
+    void close_cursor_if_needed();
+    /// Row count for select operations
+    size_t select_rowcount();
+    /// Row count for insert, delete and other non-select operations
     size_t rowcount();
 
     void set_cursor_forward_only();
+    void set_cursor_static();
 
 protected:
     bool error_is_ignored();
@@ -252,8 +312,26 @@ private:
     Sequence& operator=(const Sequence&);
 };
 
-/// Return the default repinfo file pathname
-const char* default_repinfo_file();
+static inline bool operator!=(const SQL_TIMESTAMP_STRUCT& a, const SQL_TIMESTAMP_STRUCT& b)
+{
+    return a.year != b.year || a.month != b.month || a.day != b.day || a.hour != b.hour || a.minute != b.minute || a.second != b.second || a.fraction != b.fraction;
+}
+
+std::ostream& operator<<(std::ostream& o, const SQL_TIMESTAMP_STRUCT& t);
+
+static inline SQL_TIMESTAMP_STRUCT make_sql_timestamp(int year, int month, int day, int hour, int minute, int second)
+{
+    SQL_TIMESTAMP_STRUCT res;
+    res.year = year;
+    res.month = month;
+    res.day = day;
+    res.hour = hour;
+    res.minute = minute;
+    res.second = second;
+    res.fraction = 0;
+    return res;
+}
+
 
 } // namespace db
 } // namespace dballe
