@@ -1,7 +1,7 @@
 /*
  * DB-ALLe - Archive for punctual meteorological data
  *
- * Copyright (C) 2005,2006  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,15 +22,12 @@
 #ifndef DBA_AOF_IMPORTERS_COMMON_H
 #define DBA_AOF_IMPORTERS_COMMON_H
 
-/* For round() */
-#define _ISOC99_SOURCE
-
 /*
  * Common functions for all AOF decoders.
  */
 
-#include <dballe/core/conv.h>
 #include <dballe/msg/msg.h>
+#include <dballe/msg/aof_codec.h>
 
 #include <stdio.h>
 #include <stdint.h>		/* uint32_t */
@@ -50,6 +47,8 @@
 
 #define OBS(n) (obs[n-1])
 
+namespace dballe {
+namespace msg {
 
 /* Parse a 2 bit confidence interval into a percent confidence interval */
 static inline int get_conf2(uint32_t conf)
@@ -70,39 +69,10 @@ static inline int get_conf6(uint32_t conf)
 	return get_conf2(conf >> 3);
 }
 
-/* Convert Kelvin into Celsius */
-static inline double totemp(double k) { return k / 10.0; }
-
-/* Dump a word */
-void dba_aof_dump_word(const char* prefix, uint32_t x);
-
-/* Parse latitude, longitude, date and time in the Observation Header */
-dba_err dba_aof_parse_lat_lon_datetime(dba_msg msg, const uint32_t* obs);
-
-/* Parse WMO block and station numbers in the Observation Header */
-dba_err dba_aof_parse_st_block_station(dba_msg msg, const uint32_t* obs);
-
-/* Parse string ident in the Observation Header */
-dba_err dba_aof_parse_st_ident(dba_msg msg, const uint32_t* obs);
-
-/* Parse station altitude the Observation Header */
-dba_err dba_aof_parse_altitude(dba_msg msg, const uint32_t* obs);
-
-/* Parse 27 Weather group in Synop observations */
-dba_err dba_aof_parse_weather_group(dba_msg msg, const uint32_t* obs);
-
-/* Parse 28 General cloud group in Synop observations */
-dba_err dba_aof_parse_general_cloud_group(dba_msg msg, const uint32_t* obs);
-
-/* Parse a bit-packed cloud group in Synop observations */
-dba_err dba_aof_parse_cloud_group(uint32_t val, int* ns, int* c, int* h);
-
-uint32_t dba_aof_get_extra_conf(const uint32_t* obs, int idx);
-
 /* Count the number of bits present in a word */
 static inline int count_bits(uint32_t v)
 {
-	/* Algoritmo basato su Magic Binary Numbers
+	/* Algorithm based on Magic Binary Numbers
 	 * http://graphics.stanford.edu/~seander/bithacks.html
 	const int S[] = {1, 2, 4, 8, 16}; // Magic Binary Numbers
 	const int B[] = {0x55555555, 0x33333333, 0x0F0F0F0F, 0x00FF00FF, 0x0000FFFF};
@@ -116,17 +86,38 @@ static inline int count_bits(uint32_t v)
 	return c;
 	*/
 
-	/* Algoritmo di Kernigan (1 iterazione per bit settato a 1): ci va bene
-	 * perché solitamente sono pochi */
+    // Kernigan algorithm (1 iteration per bit set to 1): better because where
+    // this function is used the bits set to 1 are usually few
 
 	unsigned int c; // c accumulates the total bits set in v
 	for (c = 0; v; c++)
-	{
 		v &= v - 1; // clear the least significant bit set
-	}
 
 	return c;
 }
 
+static inline uint32_t get_extra_conf(const uint32_t* obs, int idx)
+{
+	int count = count_bits(OBS(32));
+	uint32_t w = OBS(33+count+idx/4);
+	return (w >> ((idx % 4) * 6)) & 0x3f;
+
+}
+
+/* Convert Kelvin from AOF to Kelvins */
+static inline double totemp(double k) { return k / 10.0; }
+
+#if 0
+/* Dump a word */
+void dba_aof_dump_word(const char* prefix, uint32_t x);
+
+
+uint32_t dba_aof_get_extra_conf(const uint32_t* obs, int idx);
+
+#endif
+
+} // namespace msg
+} // namespace dballe
+
 /* vim:set ts=4 sw=4: */
 #endif
