@@ -1,7 +1,7 @@
 /*
  * dballe/msg - Hold an interpreted weather bulletin
  *
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2014  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -75,7 +75,7 @@ Msg::Msg(const Msg& m)
 {
     // Reserve space for the new contexts
     data.reserve(m.data.size());
-    
+
     // Copy the contexts
     for (vector<msg::Context*>::const_iterator i = m.data.begin();
             i != m.data.end(); ++i)
@@ -153,6 +153,11 @@ msg::Context* Msg::edit_context(const Level& lev, const Trange& tr)
     return data[pos];
 }
 
+msg::Context& Msg::obtain_station_context()
+{
+    return obtain_context(Level::ana(), Trange::ana());
+}
+
 msg::Context& Msg::obtain_context(const Level& lev, const Trange& tr)
 {
     int pos = find_index(lev, tr);
@@ -283,7 +288,7 @@ struct VarContext
         // Report type
         out << rep_memo << ",";
 
-        if (c.level.ltype1 != 257)
+        if (c.level != Level::ana())
         {
             // Datetime
             out << setfill('0') << setw(4) << (year ? year->enq(0) : 0) << "-";
