@@ -23,13 +23,17 @@
 #include <dballe/msg/context.h>
 #include <wreport/bulletin.h>
 #include <wreport/conv.h>
+#include <wreport/notes.h>
+#include <wibble/string.h>
 
 #include <cstring>
+#include <cmath>
 #include <unistd.h>
 #include <sys/types.h>
 #include <pwd.h>
 #include <fstream>
 
+using namespace wibble;
 using namespace wreport;
 using namespace std;
 
@@ -141,8 +145,7 @@ auto_ptr<Msgs> _read_msgs(const wibble::tests::Location& loc, const char* filena
 std::auto_ptr<Msgs> _read_msgs_csv(const Location& loc, const char* filename)
 {
     std::string fname = datafile(filename);
-    ifstream in(fname.c_str());
-    IstreamCSVReader reader(in);
+    CSVReader reader(fname);
 
     auto_ptr<Msgs> msgs(new Msgs);
     if (!msgs->from_csv(reader))
@@ -154,6 +157,21 @@ std::auto_ptr<Msgs> _read_msgs_csv(const Location& loc, const char* filename)
     return msgs;
 }
 
+void _export_msgs(const Location& loc, const Msgs& in, Bulletin& out, const std::string& tag, const dballe::msg::Exporter::Options& opts)
+{
+    try {
+        Encoding type = BUFR;
+        if (string(out.encoding_name()) == "CREX")
+            type = CREX;
+        std::auto_ptr<msg::Exporter> exporter(msg::Exporter::create(type, opts));
+        exporter->to_bulletin(in, out);
+    } catch (std::exception& e) {
+        dballe::tests::dump("bul-" + tag, in);
+        dballe::tests::dump("msg-" + tag, out);
+        throw tut::failure(loc.msg("exporting to bulletin (" + tag + "): " + e.what()));
+    }
+}
+
 void track_different_msgs(const Msg& msg1, const Msg& msg2, const std::string& prefix)
 {
 	string fname1 = "/tmp/test-" + prefix + "1.bufr";
@@ -270,6 +288,23 @@ void dump(const std::string& tag, const std::string& msg, const std::string& des
     cerr << desc << " saved in " << fname << endl;
 }
 
+MessageTweakers::~MessageTweakers()
+{
+    for (vector<MessageTweaker*>::iterator i = tweaks.begin(); i != tweaks.end(); ++i)
+        delete *i;
+}
+
+void MessageTweakers::add(MessageTweaker* tweak)
+{
+    tweaks.push_back(tweak);
+}
+
+void MessageTweakers::apply(Msgs& msgs)
+{
+    for (vector<MessageTweaker*>::iterator i = tweaks.begin(); i != tweaks.end(); ++i)
+        (*i)->tweak(msgs);
+}
+
 namespace tweaks {
 
 void StripAttrs::tweak(Msgs& msgs)
@@ -511,6 +546,25 @@ void RoundGeopotential::tweak(Msgs& msgs)
     }
 }
 
+HeightToGeopotential::HeightToGeopotential()
+{
+    table = Vartable::get("B0000000000000014000");
+}
+void HeightToGeopotential::tweak(Msgs& msgs)
+{
+    for (Msgs::iterator mi = msgs.begin(); mi != msgs.end(); ++mi)
+    {
+        Msg& m = **mi;
+        for (vector<msg::Context*>::iterator ci = m.data.begin(); ci != m.data.end(); ++ci)
+        {
+            msg::Context& c = **ci;
+            if (c.level.ltype1 != 102) continue;
+            Var var(table->query(WR_VAR(0, 10, 8)), round(c.level.l1 * 9.807 / 10) * 10);
+            c.set(var);
+        }
+    }
+}
+
 void RoundVSS::tweak(Msgs& msgs)
 {
     for (Msgs::iterator mi = msgs.begin(); mi != msgs.end(); ++mi)
@@ -525,6 +579,203 @@ void RoundVSS::tweak(Msgs& msgs)
     }
 }
 
+RemoveContext::RemoveContext(const Level& lev, const Trange& tr)
+    : lev(lev), tr(tr)
+{
+}
+void RemoveContext::tweak(Msgs& msgs)
+{
+    for (Msgs::iterator mi = msgs.begin(); mi != msgs.end(); ++mi)
+    {
+        Msg& m = **mi;
+        m.remove_context(lev, tr);
+    }
+}
+
+}
+
+TestMessage::TestMessage(Encoding type, const std::string& name)
+    : name(name), type(type), bulletin(0)
+{
+    switch (type)
+    {
+        case BUFR: bulletin = BufrBulletin::create().release(); break;
+        case CREX: bulletin = CrexBulletin::create().release(); break;
+        default: throw wreport::error_unimplemented("Unsupported message type");
+    }
+}
+
+TestMessage::~TestMessage()
+{
+    if (bulletin) delete bulletin;
+}
+
+void TestMessage::read_from_file(const std::string& fname, const msg::Importer::Options& input_opts)
+{
+    auto_ptr<Rawmsg> src = read_rawmsg(fname.c_str(), type);
+    read_from_raw(*src, input_opts);
+}
+
+void TestMessage::read_from_raw(const Rawmsg& msg, const msg::Importer::Options& input_opts)
+{
+    std::auto_ptr<msg::Importer> importer(msg::Importer::create(type, input_opts));
+    raw = msg;
+    bulletin->decode(raw);
+    importer->from_rawmsg(raw, msgs);
+}
+
+void TestMessage::read_from_msgs(const Msgs& _msgs, const msg::Exporter::Options& export_opts)
+{
+    // Export
+    std::auto_ptr<msg::Exporter> exporter(msg::Exporter::create(type, export_opts));
+
+    msgs = _msgs;
+    exporter->to_bulletin(msgs, *bulletin);
+    bulletin->encode(raw);
+}
+
+TestCodec::TestCodec(const std::string& fname, Encoding type)
+    : fname(fname), type(type), verbose(false), expected_subsets(1), expected_min_vars(1)
+{
+}
+
+void TestCodec::configure_ecmwf_to_wmo_tweaks()
+{
+    after_convert_import.add(new tweaks::StripQCAttrs);
+}
+
+void TestCodec::do_compare(const dballe::tests::Location& loc, const TestMessage& msg1, const TestMessage& msg2)
+{
+    // Compare msgs
+    {
+        stringstream str;
+        notes::Collect c(str);
+        int diffs = msg1.msgs.diff(msg2.msgs);
+        if (diffs)
+        {
+            dballe::tests::dump("msg1", msg1.msgs);
+            dballe::tests::dump("msg2", msg2.msgs);
+            dballe::tests::dump("msg", msg2.raw);
+            dballe::tests::dump("diffs", str.str(), "details of differences");
+            throw tut::failure(loc.msg(str::fmtf("found %d differences", diffs)));
+        }
+    }
+
+    // Compare bulletins
+    try {
+        if (msg2.bulletin->subsets.size() != (unsigned)expected_subsets)
+            throw tut::failure(loc.msg(str::fmtf("Number of subsets differ from expected: %zd != %d\n", msg2.bulletin->subsets.size(), expected_subsets)));
+        if (msg2.bulletin->subsets[0].size() < (unsigned)expected_min_vars)
+            throw tut::failure(loc.msg(str::fmtf("Number of items in first subset is too small: %zd < %d\n", msg2.bulletin->subsets[0].size(), expected_min_vars)));
+    } catch (...) {
+        dballe::tests::dump("bull1", *msg1.bulletin);
+        dballe::tests::dump("bull2", *msg2.bulletin);
+        dballe::tests::dump("msg", msg2.raw);
+        throw;
+    }
+}
+
+void TestCodec::run_reimport(const dballe::tests::Location& loc)
+{
+    if (verbose) cerr << "Running test " << loc.locstr() << endl;
+
+    // Import
+    if (verbose) cerr << "Importing " << fname << " " << input_opts.to_string() << endl;
+    TestMessage orig(type, "orig");
+    try {
+        orig.read_from_file(fname, input_opts);
+    } catch (std::exception& e) {
+        throw tut::failure(loc.msg(string("cannot decode ") + fname + ": " + e.what()));
+    }
+
+    inner_ensure(orig.msgs.size() > 0);
+
+    // Run tweaks
+    after_reimport_import.apply(orig.msgs);
+
+    // Export
+    if (verbose) cerr << "Exporting " << output_opts.to_string() << endl;
+    TestMessage exported(type, "exported");
+    try {
+        exported.read_from_msgs(orig.msgs, output_opts);
+    } catch (std::exception& e) {
+        //dballe::tests::dump("bul1", *exported);
+        //balle::tests::dump("msg1", *msgs1);
+        throw tut::failure(loc.msg(string("cannot export: ") + e.what()));
+    }
+
+    // Import again
+    TestMessage final(type, "final");
+    try {
+        final.read_from_raw(exported.raw, input_opts);
+    } catch (std::exception& e) {
+        //dballe::tests::dump("msg1", *msgs1);
+        //dballe::tests::dump("msg", rawmsg);
+        throw tut::failure(loc.msg(string("importing from exported rawmsg: ") + e.what()));
+    }
+
+    // Run tweaks
+    after_reimport_reimport.apply(final.msgs);
+
+    try {
+        do_compare(loc, orig, final);
+    } catch (...) {
+        dballe::tests::dump("reexported", exported.raw);
+        throw;
+    }
+}
+
+void TestCodec::run_convert(const dballe::tests::Location& loc, const std::string& tplname)
+{
+    if (verbose) cerr << "Running test " << loc.locstr() << endl;
+
+    // Import
+    if (verbose) cerr << "Importing " << fname << " " << input_opts.to_string() << endl;
+    TestMessage orig(type, "orig");
+    try {
+        orig.read_from_file(fname, input_opts);
+    } catch (std::exception& e) {
+        throw tut::failure(loc.msg(string("cannot decode ") + fname + ": " + e.what()));
+    }
+
+    inner_ensure(orig.msgs.size() > 0);
+
+    // Run tweaks
+    after_convert_import.apply(orig.msgs);
+
+    // Export
+    if (verbose) cerr << "Exporting " << output_opts.to_string() << endl;
+    msg::Exporter::Options output_opts = this->output_opts;
+    output_opts.template_name = tplname;
+    TestMessage exported(type, "exported");
+    try {
+        exported.read_from_msgs(orig.msgs, output_opts);
+    } catch (std::exception& e) {
+        //dballe::tests::dump("bul1", *exported);
+        //balle::tests::dump("msg1", *msgs1);
+        throw tut::failure(loc.msg(string("cannot export: ") + e.what()));
+    }
+
+    // Import again
+    TestMessage final(type, "final");
+    try {
+        final.read_from_raw(exported.raw, input_opts);
+    } catch (std::exception& e) {
+        //dballe::tests::dump("msg1", *msgs1);
+        //dballe::tests::dump("msg", rawmsg);
+        throw tut::failure(loc.msg(string("importing from exported rawmsg: ") + e.what()));
+    }
+
+    // Run tweaks
+    after_convert_reimport_on_orig.apply(orig.msgs);
+    after_convert_reimport.apply(final.msgs);
+
+    try {
+        do_compare(loc, orig, final);
+    } catch (...) {
+        dballe::tests::dump("converted", exported.raw);
+        throw;
+    }
 }
 
 }
