@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2013  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,262 +48,7 @@ struct wr_export_shar
 };
 TESTGRP(wr_export);
 
-// Common machinery for import -> export -> reimport tests
-template<class B>
-struct ReimportTest
-{
-    typedef dballe::tests::MessageTweaker Tweaker;
-
-    string fname;
-    Encoding type;
-    auto_ptr<Msgs> msgs1;
-    auto_ptr<Msgs> msgs2;
-    msg::Importer::Options input_opts;
-    msg::Exporter::Options output_opts;
-    vector<Tweaker*> tweaks;
-    vector<Tweaker*> ecmwf_tweaks;
-    vector<Tweaker*> wmo_tweaks;
-    bool do_ecmwf_tweaks;
-    bool do_wmo_tweaks;
-    bool do_ignore_context_attrs;
-    bool do_round_geopotential;
-    bool verbose;
-
-    void clear_tweaks()
-    {
-        for (typename vector<Tweaker*>::iterator i = tweaks.begin();
-                i != tweaks.end(); ++i)
-            delete *i;
-        tweaks.clear();
-        for (typename vector<Tweaker*>::iterator i = ecmwf_tweaks.begin();
-                i != ecmwf_tweaks.end(); ++i)
-            delete *i;
-        ecmwf_tweaks.clear();
-        for (typename vector<Tweaker*>::iterator i = wmo_tweaks.begin();
-                i != wmo_tweaks.end(); ++i)
-            delete *i;
-        wmo_tweaks.clear();
-    }
-
-    ReimportTest(const std::string& fname, Encoding type=BUFR)
-        : fname(fname), type(type), do_ecmwf_tweaks(false), do_wmo_tweaks(false),
-          do_ignore_context_attrs(false), do_round_geopotential(false), verbose(false)
-    {
-        ecmwf_tweaks.push_back(new StripQCAttrs());
-        wmo_tweaks.push_back(new RoundLegacyVars());
-    }
-    ~ReimportTest()
-    {
-        clear_tweaks();
-    }
-
-    void do_test(const dballe::tests::Location& loc, const char* tname1, const char* tname2=NULL)
-    {
-        if (verbose) cerr << "Running test " << loc.locstr() << endl;
-
-        std::auto_ptr<msg::Importer> importer(msg::Importer::create(type, input_opts));
-
-        // Import
-        if (verbose) cerr << "Importing " << fname << " " << input_opts.to_string() << endl;
-        msgs1 = inner_read_msgs_opts(fname.c_str(), type, input_opts);
-        inner_ensure(msgs1->size() > 0);
-
-        // Run tweaks
-        for (typename vector<Tweaker*>::iterator i = tweaks.begin(); i != tweaks.end(); ++i)
-        {
-            if (verbose) cerr << "Running tweak " << (*i)->desc() << endl;
-            (*i)->tweak(*msgs1);
-        }
-        if (do_ecmwf_tweaks)
-            for (typename vector<Tweaker*>::iterator i = ecmwf_tweaks.begin(); i != ecmwf_tweaks.end(); ++i)
-            {
-                if (verbose) cerr << "Running ecmwf tweak " << (*i)->desc() << endl;
-                (*i)->tweak(*msgs1);
-            }
-        if (do_wmo_tweaks)
-            for (typename vector<Tweaker*>::iterator i = wmo_tweaks.begin(); i != wmo_tweaks.end(); ++i)
-            {
-                if (verbose) cerr << "Running wmo tweak " << (*i)->desc() << endl;
-                (*i)->tweak(*msgs1);
-            }
-
-        // Export
-        auto_ptr<Bulletin> bulletin(B::create());
-        try {
-            if (tname1 != NULL)
-                output_opts.template_name = tname1;
-            else
-                output_opts.template_name.clear();
-            if (verbose) cerr << "Exporting " << output_opts.to_string() << endl;
-            std::auto_ptr<msg::Exporter> exporter(msg::Exporter::create(type, output_opts));
-            exporter->to_bulletin(*msgs1, *bulletin);
-        } catch (std::exception& e) {
-            dballe::tests::dump("bul1", *bulletin);
-            dballe::tests::dump("msg1", *msgs1);
-            throw tut::failure(loc.msg(string("exporting to bulletin (first template): ") + e.what()));
-        }
-
-        // Encode
-        Rawmsg rawmsg;
-        try {
-            bulletin->encode(rawmsg);
-            //exporter->to_rawmsg(*msgs1, rawmsg);
-        } catch (std::exception& e) {
-            dballe::tests::dump("bul1", *bulletin);
-            dballe::tests::dump("msg1", *msgs1);
-            throw tut::failure(loc.msg(string("encoding to rawmsg (first template): ") + e.what()));
-        }
-
-        // Import again
-        if (verbose) cerr << "Reimporting " << input_opts.to_string() << endl;
-        msgs2.reset(new Msgs);
-        try {
-            importer->from_rawmsg(rawmsg, *msgs2);
-        } catch (std::exception& e) {
-            dballe::tests::dump("msg1", *msgs1);
-            dballe::tests::dump("msg", rawmsg);
-            throw tut::failure(loc.msg(string("importing from rawmsg (first template): ") + e.what()));
-        }
-
-        auto_ptr<Msgs> msgs3;
-        if (tname2)
-        {
-            // Export
-            auto_ptr<Bulletin> bulletin(B::create());
-            try {
-                output_opts.template_name = tname2;
-                if (verbose) cerr << "Reexporting " << output_opts.to_string() << endl;
-                std::auto_ptr<msg::Exporter> exporter(msg::Exporter::create(type, output_opts));
-                exporter->to_bulletin(*msgs2, *bulletin);
-            } catch (std::exception& e) {
-                dballe::tests::dump("bul2", *bulletin);
-                dballe::tests::dump("msg2", *msgs1);
-                throw tut::failure(loc.msg(string("exporting to bulletin (second template): ") + e.what()));
-            }
-
-            // Encode
-            rawmsg.clear();
-            try {
-                bulletin->encode(rawmsg);
-                //exporter->to_rawmsg(*msgs1, rawmsg);
-            } catch (std::exception& e) {
-                dballe::tests::dump("bul2", *bulletin);
-                dballe::tests::dump("msg2", *msgs1);
-                throw tut::failure(loc.msg(string("encoding to rawmsg (second template): ") + e.what()));
-            }
-
-            // Import again
-            msgs3.reset(new Msgs);
-            try {
-                if (verbose) cerr << "Reimporting " << input_opts.to_string() << endl;
-                importer->from_rawmsg(rawmsg, *msgs3);
-            } catch (std::exception& e) {
-                dballe::tests::dump("msg2", *msgs2);
-                dballe::tests::dump("raw2", rawmsg);
-                throw tut::failure(loc.msg(string("importing from rawmsg (first template): ") + e.what()));
-            }
-        } else
-            msgs3 = msgs2;
-
-#if 0
-        // Run tweaks
-        for (typename vector<Tweaker*>::iterator i = tweaks.begin(); i != tweaks.end(); ++i)
-            (*i)->clean_second(*msgs3);
-#endif
-        if (do_ignore_context_attrs)
-        {
-            StripContextAttrs sca;
-            sca.tweak(*msgs1);
-        }
-        if (do_round_geopotential)
-        {
-            RoundGeopotential rg;
-            rg.tweak(*msgs1);
-            rg.tweak(*msgs3);
-        }
-
-        // Compare
-        stringstream str;
-        notes::Collect c(str);
-        int diffs = msgs1->diff(*msgs3);
-        if (diffs)
-        {
-            dballe::tests::dump("msg1", *msgs1);
-            if (msgs2.get())
-                dballe::tests::dump("msg2", *msgs2);
-            dballe::tests::dump("msg3", *msgs3);
-            dballe::tests::dump("msg", rawmsg);
-            dballe::tests::dump("diffs", str.str(), "details of differences");
-            throw tut::failure(loc.msg(str::fmtf("found %d differences", diffs)));
-        }
-    }
-
-#define inner_do_test(name, ...) do_test(wibble::tests::Location(loc, __FILE__, __LINE__, name), __VA_ARGS__)
-    void do_ecmwf(const dballe::tests::Location& loc, const char* template_type="synop")
-    {
-        string ecmwf_template_name = string(template_type) + "-ecmwf";
-        string wmo_template_name = string(template_type) + "-wmo";
-        do_wmo_tweaks = false;
-
-        input_opts.simplified = true;
-        do_round_geopotential = false;
-
-        do_ecmwf_tweaks = false;
-        inner_do_test("simp-ecmwf-through-auto", NULL);
-        inner_do_test("simp-ecmwf-through-ecmwf", ecmwf_template_name.c_str());
-
-        do_ecmwf_tweaks = true;
-        do_round_geopotential = true;
-        inner_do_test("simp-ecmwf-through-wmo", wmo_template_name.c_str());
-
-        input_opts.simplified = false;
-        do_round_geopotential = false;
-
-        do_ecmwf_tweaks = false;
-        inner_do_test("real-ecmwf-through-auto", NULL);
-        inner_do_test("real-ecmwf-through-ecmwf", ecmwf_template_name.c_str());
-
-        do_ecmwf_tweaks = true;
-        do_round_geopotential = true;
-        inner_do_test("real-ecmwf-through-wmo", wmo_template_name.c_str());
-    }
-    void do_wmo(const dballe::tests::Location& loc, const char* template_type="synop")
-    {
-        string ecmwf_template_name = string(template_type) + "-ecmwf";
-        string wmo_template_name = string(template_type) + "-wmo";
-        do_ecmwf_tweaks = false;
-
-        input_opts.simplified = true;
-        do_round_geopotential = false;
-
-        do_wmo_tweaks = false;
-        inner_do_test("simp-wmo-through-auto", NULL);
-        inner_do_test("simp-wmo-through-wmo", wmo_template_name.c_str());
-
-        do_wmo_tweaks = true;
-        do_ignore_context_attrs = true;
-        do_round_geopotential = true;
-        inner_do_test("simp-wmo-through-ecmwf", ecmwf_template_name.c_str());
-        do_ignore_context_attrs = false;
-
-        input_opts.simplified = false;
-        do_round_geopotential = false;
-
-        do_wmo_tweaks = false;
-        inner_do_test("real-wmo-through-auto", NULL);
-        inner_do_test("real-wmo-through-wmo", wmo_template_name.c_str());
-
-        // There doesn't seem much sense testing this at the moment
-        //do_tweaks = true;
-        //inner_do_test("real-wmo-through-ecmwf", ecmwf_template_name.c_str());
-    }
-#undef inner_do_test
-};
-typedef ReimportTest<BufrBulletin> BufrReimportTest;
-typedef ReimportTest<CrexBulletin> CrexReimportTest;
-#define run_test(obj, meth, ...) obj.meth(wibble::tests::Location(__FILE__, __LINE__, (obj.fname + " " #__VA_ARGS__).c_str()), __VA_ARGS__)
-
-
+#include "wr_export.h"
 
 // Test that plain re-export of all our BUFR test files is possible
 template<> template<>
@@ -426,8 +171,15 @@ void to::test<3>()
 template<> template<>
 void to::test<5>()
 {
-    BufrReimportTest test("bufr/obs0-1.22.bufr");
-    run_test(test, do_ecmwf, "synop");
+    dballe::tests::TestCodec test("bufr/obs0-1.22.bufr");
+    test.expected_min_vars = 34;
+    test.configure_ecmwf_to_wmo_tweaks();
+
+    TEST_reimport(test);
+    TEST_convert(test, "synop-wmo");
+
+    BufrReimportTest test1("bufr/obs0-1.22.bufr");
+    run_test(test1, do_ecmwf, "synop");
 }
 template<> template<>
 void to::test<6>()
@@ -449,6 +201,9 @@ void to::test<8>()
     BufrReimportTest test("bufr/synop-cloudbelow.bufr");
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 1);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<9>()
@@ -456,6 +211,9 @@ void to::test<9>()
     BufrReimportTest test("bufr/synop-evapo.bufr");
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 2);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<10>()
@@ -463,6 +221,9 @@ void to::test<10>()
     BufrReimportTest test("bufr/synop-groundtemp.bufr");
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 1);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<11>()
@@ -471,6 +232,9 @@ void to::test<11>()
     test.tweaks.push_back(new TruncStName());
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 1);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<12>()
@@ -478,6 +242,9 @@ void to::test<12>()
     BufrReimportTest test("bufr/synop-oddgust.bufr");
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 0);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<13>()
@@ -486,6 +253,9 @@ void to::test<13>()
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     test.wmo_tweaks.push_back(new RemoveSynopWMOOddprec());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 0);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<14>()
@@ -493,6 +263,9 @@ void to::test<14>()
     BufrReimportTest test("bufr/synop-strayvs.bufr");
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 1);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<15>()
@@ -500,6 +273,9 @@ void to::test<15>()
     BufrReimportTest test("bufr/synop-sunshine.bufr");
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 1);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 
 template<> template<>
@@ -508,6 +284,9 @@ void to::test<16>()
     BufrReimportTest test("bufr/synop-gtscosmo.bufr");
     test.wmo_tweaks.push_back(new RemoveSynopWMOOnlyVars());
     run_test(test, do_wmo, "synop");
+    ensure_equals(test.exported->type, 0);
+    ensure_equals(test.exported->subtype, 2);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 
 // Re-export test for old style temps
@@ -518,6 +297,7 @@ void to::test<17>()
     test.ecmwf_tweaks.push_back(new RemoveTempWMOOnlyVars());
     run_test(test, do_ecmwf, "temp");
 }
+
 template<> template<>
 void to::test<18>()
 {
@@ -525,11 +305,12 @@ void to::test<18>()
     test.ecmwf_tweaks.push_back(new RemoveTempWMOOnlyVars());
     run_test(test, do_ecmwf, "temp");
 }
+
 template<> template<>
 void to::test<19>()
 {
     BufrReimportTest test("bufr/obs2-91.2.bufr");
-    run_test(test, do_ecmwf, "temp");
+    run_test(test, do_ecmwf, "pilot");
 }
 template<> template<>
 void to::test<20>()
@@ -581,6 +362,9 @@ void to::test<24>()
     test.wmo_tweaks.push_back(new RemoveTempWMOOnlyVars());
     test.wmo_tweaks.push_back(new RoundVSS());
     run_test(test, do_wmo, "temp");
+    ensure_equals(test.exported->type, 2);
+    ensure_equals(test.exported->subtype, 4);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<25>()
@@ -588,6 +372,9 @@ void to::test<25>()
     BufrReimportTest test("bufr/temp-gts2.bufr");
     test.wmo_tweaks.push_back(new RemoveTempWMOOnlyVars());
     run_test(test, do_wmo, "temp");
+    ensure_equals(test.exported->type, 2);
+    ensure_equals(test.exported->subtype, 4);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<26>()
@@ -595,6 +382,9 @@ void to::test<26>()
     BufrReimportTest test("bufr/temp-gts3.bufr");
     test.wmo_tweaks.push_back(new RemoveTempWMOOnlyVars());
     run_test(test, do_wmo, "temp");
+    ensure_equals(test.exported->type, 2);
+    ensure_equals(test.exported->subtype, 4);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<27>()
@@ -603,6 +393,9 @@ void to::test<27>()
     //test.verbose = true;
     test.wmo_tweaks.push_back(new RemoveTempWMOOnlyVars());
     run_test(test, do_wmo, "temp");
+    ensure_equals(test.exported->type, 2);
+    ensure_equals(test.exported->subtype, 4);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<28>()
@@ -645,6 +438,9 @@ void to::test<31>()
 {
     BufrReimportTest test("bufr/temp-bad4.bufr");
     run_test(test, do_test, "temp-wmo");
+    ensure_equals(test.exported->type, 2);
+    ensure_equals(test.exported->subtype, 4);
+    ensure_equals(test.exported->localsubtype, 255);
 }
 template<> template<>
 void to::test<32>()
@@ -832,6 +628,157 @@ void to::test<40>()
         throw tut::failure(str::fmtf("%zd/%d errors:\n", fails.size(), i) + str::join(fails.begin(), fails.end(), "\n"));
 }
 
+template<> template<>
+void to::test<41>()
+{
+    BufrReimportTest test("bufr/pilot-gts1.bufr");
+    run_test(test, do_test, "pilot-wmo");
+
+    ensure_equals(test.exported->type, 2);
+    ensure_equals(test.exported->subtype, 1);
+    ensure_equals(test.exported->localsubtype, 255);
+}
+
+template<> template<>
+void to::test<42>()
+{
+    BufrReimportTest test("bufr/pilot-gts1.bufr");
+    run_test(test, do_test, "pilot-wmo");
+
+    ensure_equals(test.exported->type, 2);
+    ensure_equals(test.exported->subtype, 1);
+    ensure_equals(test.exported->localsubtype, 255);
+}
+
+template<> template<>
+void to::test<43>()
+{
+    BufrReimportTest test("bufr/pilot-ecmwf-geopotential.bufr");
+    test.tweaks.push_back(new StripQCAttrs());
+    StripVars* sv = new StripVars();
+    sv->codes.push_back(WR_VAR(0, 10, 8));
+    test.tweaks.push_back(sv);
+    run_test(test, do_test, "pilot-wmo");
+    //run_test(test, do_wmo, "pilot");
+}
+
+// Test for a bug where geopotential levels became pressure levels
+template<> template<>
+void to::test<44>()
+{
+    auto_ptr<Msgs> msgs1 = read_msgs("bufr/pilot-ecmwf-geopotential.bufr", BUFR);
+    ensure_equals(msgs1->size(), 1);
+    Msg& msg1 = *(*msgs1)[0];
+
+    // Geopotential levels are converted to height above msl
+    const msg::Context* c = msg1.find_context(Level(102, 900), Trange(254, 0, 0));
+    ensure(c != NULL);
+
+    // Convert to WMO template
+    msg::Exporter::Options output_opts;
+    output_opts.template_name = "pilot-wmo";
+    //if (verbose) cerr << "Exporting " << output_opts.to_string() << endl;
+    std::auto_ptr<BufrBulletin> bulletin = BufrBulletin::create();
+    test_export_msgs(*msgs1, *bulletin, "towmo", output_opts);
+
+    // Import again
+    Msgs msgs2;
+    std::auto_ptr<msg::Importer> imp = msg::Importer::create(BUFR);
+    imp->from_bulletin(*bulletin, msgs2);
+    ensure_equals(msgs2.size(), 1);
+    Msg& msg2 = *msgs2[0];
+
+    // Ensure we didn't get pressure levels
+    ensure(msg2.find_context(Level(100, 900), Trange(254, 0, 0)) == NULL);
+}
+
+// Test for a range error in one specific BUFR
+template<> template<>
+void to::test<45>()
+{
+    auto_ptr<Msgs> msgs1 = read_msgs("bufr/temp-2-255.bufr", BUFR);
+    ensure_equals(msgs1->size(), 1);
+    Msg& msg1 = *(*msgs1)[0];
+
+    // Convert to CREX
+    msg::Exporter::Options output_opts;
+    output_opts.template_name = "temp-wmo";
+    std::auto_ptr<CrexBulletin> bulletin = CrexBulletin::create();
+    test_export_msgs(*msgs1, *bulletin, "tocrex", output_opts);
+
+    // Import again
+    Msgs msgs2;
+    std::auto_ptr<msg::Importer> imp = msg::Importer::create(BUFR);
+    imp->from_bulletin(*bulletin, msgs2);
+    ensure_equals(msgs2.size(), 1);
+    Msg& msg2 = *msgs2[0];
+}
+
+// Test that temp ship subtype is set correctly
+template<> template<>
+void to::test<46>()
+{
+    std::auto_ptr<Msgs> msgs = read_msgs("bufr/obs2-102.1.bufr", BUFR);
+    auto_ptr<Bulletin> bulletin(BufrBulletin::create());
+    msg::Exporter::Options opts;
+    opts.template_name = "temp-wmo";
+    test_export_msgs(*msgs, *bulletin, "tempship", opts);
+    ensure_equals(bulletin->type, 2);
+    ensure_equals(bulletin->subtype, 5);
+    ensure_equals(bulletin->localsubtype, 255);
+}
+
+// Test that pilot subtype is set correctly
+template<> template<>
+void to::test<47>()
+{
+    std::auto_ptr<Msgs> msgs = read_msgs("bufr/obs2-91.2.bufr", BUFR);
+    auto_ptr<Bulletin> bulletin(BufrBulletin::create());
+    msg::Exporter::Options opts;
+    opts.template_name = "pilot-wmo";
+    test_export_msgs(*msgs, *bulletin, "pilotwmo", opts);
+    ensure_equals(bulletin->type, 2);
+    ensure_equals(bulletin->subtype, 1);
+    ensure_equals(bulletin->localsubtype, 255);
+}
+
+// Test conversion from amdar ECMWF to amdar WMO
+template<> template<>
+void to::test<48>()
+{
+    BufrReimportTest test("bufr/ecmwf-amdar1.bufr");
+    test.tweaks.push_back(new StripQCAttrs());
+    run_test(test, do_test, "amdar");
+}
+
+// Test conversion from acars ECMWF to amdar WMO
+template<> template<>
+void to::test<49>()
+{
+    BufrReimportTest test("bufr/ecmwf-acars1.bufr");
+    test.tweaks.push_back(new StripQCAttrs());
+    run_test(test, do_test, "acars");
+}
+
+// Test that temp vad subtype is set correctly
+template<> template<>
+void to::test<50>()
+{
+    BufrReimportTest test("bufr/vad.bufr");
+    //test.tweaks.push_back(new StripQCAttrs());
+    run_test(test, do_test, "temp");
+#if 0
+    std::auto_ptr<Msgs> msgs = read_msgs("bufr/vad.bufr", BUFR);
+    auto_ptr<Bulletin> bulletin(BufrBulletin::create());
+    msg::Exporter::Options opts;
+    opts.template_name = "temp";
+    test_export_msgs(*msgs, *bulletin, "temp", opts);
+    ensure_equals(bulletin->type, 6);
+    ensure_equals(bulletin->subtype, 1);
+    ensure_equals(bulletin->localsubtype, 255);
+#endif
+}
+
 }
 
 /* vim:set ts=4 sw=4: */
