@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2013  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "msg/msgs.h"
 #include "msg/context.h"
 #include <wreport/bulletin.h>
+#include <wreport/options.h>
 #include <cstring>
 
 using namespace dballe;
@@ -392,6 +393,93 @@ void to::test<21>()
     IS(ident, "EU7866");
 }
 
+// BUFR that has a variable that goes out of range when converted to local B
+// table
+template<> template<>
+void to::test<22>()
+{
+    try
+    {
+        // Read and interpretate the message
+        std::auto_ptr<Rawmsg> raw = read_rawmsg("bufr/interpreted-range.bufr", BUFR);
+        std::auto_ptr<msg::Importer> importer = msg::Importer::create(BUFR);
+        std::auto_ptr<Msgs> msgs(new Msgs);
+        importer->from_rawmsg(*raw, *msgs);
+        ensure(false);
+    } catch (wreport::error_domain& e) {
+        //cerr << e.code() << "--" << e.what() << endl;
+    }
+
+    {
+        wreport::options::LocalOverride<bool> o(wreport::options::var_silent_domain_errors, true);
+        auto_ptr<Msgs> msgs = read_msgs("bufr/interpreted-range.bufr", BUFR);
+        ensure_equals(msgs->size(), 1u);
+        const Msg& msg = *(*msgs)[0];
+        ensure_equals(msg.type, MSG_SHIP);
+        IS(ident, "DBBC");
+    }
+}
+
+// WMO PILOT, with geopotential levels
+template<> template<>
+void to::test<23>()
+{
+    auto_ptr<Msgs> msgs = read_msgs("bufr/pilot-gts1.bufr", BUFR);
+    ensure_equals(msgs->size(), 1u);
+    const Msg& msg = *(*msgs)[0];
+    ensure_equals(msg.type, MSG_PILOT);
+}
+
+// WMO PILOT, with pressure levels
+template<> template<>
+void to::test<24>()
+{
+    auto_ptr<Msgs> msgs = read_msgs("bufr/pilot-gts2.bufr", BUFR);
+    ensure_equals(msgs->size(), 1u);
+    const Msg& msg = *(*msgs)[0];
+    ensure_equals(msg.type, MSG_PILOT);
+}
+
+// WMO PILOT, with pressure levels
+template<> template<>
+void to::test<25>()
+{
+    // FIXME: this still fails
+    auto_ptr<Msgs> msgs = read_msgs("bufr/temp-tsig-2.bufr", BUFR);
+    ensure_equals(msgs->size(), 1u);
+    const Msg& msg = *(*msgs)[0];
+    ensure_equals(msg.type, MSG_TEMP);
+}
+
+// WMO pilot pressure
+template<> template<>
+void to::test<26>()
+{
+    auto_ptr<Msgs> msgs = read_msgs("bufr/pilot-gts3.bufr", BUFR);
+    ensure_equals(msgs->size(), 1u);
+    const Msg& msg = *(*msgs)[0];
+    ensure_equals(msg.type, MSG_PILOT);
+}
+
+// WMO pilot geopotential
+template<> template<>
+void to::test<27>()
+{
+    auto_ptr<Msgs> msgs = read_msgs("bufr/pilot-gts4.bufr", BUFR);
+    ensure_equals(msgs->size(), 1u);
+    const Msg& msg = *(*msgs)[0];
+    ensure_equals(msg.type, MSG_PILOT);
+}
+
+
+template<> template<>
+void to::test<28>()
+{
+    auto_ptr<Msgs> msgs = read_msgs("bufr/vad.bufr", BUFR);
+    ensure_equals(msgs->size(), 1u);
+    const Msg& msg = *(*msgs)[0];
+    ensure_equals(msg.type, MSG_TEMP);
+}
 #if 0
 /* Check that a BUFR from a synop high-level station correctly reports isobaric
  * surface and geopotential */
