@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2011  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2012  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <wreport/bulletin.h>
 #include <wreport/conv.h>
 #include <wreport/codetables.h>
+#include <wreport/notes.h>
 #include "msg/msgs.h"
 #include "msg/context.h"
 
@@ -45,8 +46,17 @@ using namespace std;
 #define TEMP_ECMWF_SHIP_NAME "temp-ecmwf-ship"
 #define TEMP_ECMWF_SHIP_DESC "Temp ECMWF ship (2.102)"
 
+#define TEMP_RADAR_NAME "temp-radar"
+#define TEMP_RADAR_DESC "Temp radar doppler wind profile (6.1)"
+
 #define PILOT_NAME "pilot"
-#define PILOT_DESC "Pilot (2.91)"
+#define PILOT_DESC "pilot (autodetect)"
+
+#define PILOT_WMO_NAME "pilot-wmo"
+#define PILOT_WMO_DESC "Pilot (2.1, 2.2, 2.3)"
+
+#define PILOT_ECMWF_NAME "pilot-ecmwf"
+#define PILOT_ECMWF_DESC "Pilot (2.91)"
 
 
 namespace dballe {
@@ -55,7 +65,7 @@ namespace wr {
 
 namespace {
 
-// Base template for ships
+// Base template for vertical soundings
 struct TempBase : public Template
 {
     bool is_crex;
@@ -166,6 +176,36 @@ struct TempBase : public Template
     {
         Template::to_subset(msg, subset);
     }
+
+    bool do_D03051(const msg::Context& c)
+    {
+        add(WR_VAR(0,  4, 86), &c);
+        add(WR_VAR(0,  8, 42), &c);
+        if (c.level.l1 == MISSING_INT)
+            subset->store_variable_undef(WR_VAR(0, 7, 4));
+        else
+            subset->store_variable_d(WR_VAR(0, 7, 4), c.level.l1);
+        add(WR_VAR(0,  5, 15), &c);
+        add(WR_VAR(0,  6, 15), &c);
+        add(WR_VAR(0, 11, 61), &c);
+        add(WR_VAR(0, 11, 62), &c);
+        return true;
+    }
+
+    bool do_D03053(const msg::Context& c)
+    {
+        add(WR_VAR(0,  4, 86), &c);
+        add(WR_VAR(0,  8, 42), &c);
+        if (c.level.l1 == MISSING_INT)
+            subset->store_variable_undef(WR_VAR(0, 7, 9));
+        else
+            subset->store_variable_d(WR_VAR(0, 7, 9), c.level.l1);
+        add(WR_VAR(0,  5, 15), &c);
+        add(WR_VAR(0,  6, 15), &c);
+        add(WR_VAR(0, 11, 61), &c);
+        add(WR_VAR(0, 11, 62), &c);
+        return true;
+    }
 };
 
 struct TempWMO : public TempBase
@@ -180,6 +220,19 @@ struct TempWMO : public TempBase
     {
         TempBase::setupBulletin(bulletin);
 
+        bulletin.type = 2;
+        bulletin.subtype = 4;
+        bulletin.localsubtype = 255;
+        for (Msgs::const_iterator mi = msgs.begin(); mi != msgs.end(); ++mi)
+        {
+            const Msg& msg = **mi;
+            if (msg.type == MSG_TEMP_SHIP)
+            {
+                bulletin.subtype = 5;
+                break;
+            }
+        }
+
         // Data descriptor section
         bulletin.datadesc.clear();
         bulletin.datadesc.push_back(WR_VAR(3, 9, 52));
@@ -213,7 +266,7 @@ struct TempWMO : public TempBase
                 if (const Var* var = c.find(WR_VAR(0, 10, 8)))
                     add(WR_VAR(0, 10, 9), var);
                 else
-                    subset->store_variable_d(WR_VAR(0, 10, 9), (double)c.level.l1 * 9.80665);
+                    subset->store_variable_d(WR_VAR(0, 10, 9), (double)c.level.l1);
                 break;
             case 103:
                 add(WR_VAR(0, 7, 4), &c, WR_VAR(0, 10, 4));
@@ -228,21 +281,6 @@ struct TempWMO : public TempBase
         add(WR_VAR(0, 11,   2), &c);
     }
 
-    bool do_D03051(const msg::Context& c)
-    {
-        add(WR_VAR(0,  4, 86), &c);
-        add(WR_VAR(0,  8, 42), &c);
-        if (c.level.l1 == MISSING_INT)
-            subset->store_variable_undef(WR_VAR(0, 7, 4));
-        else
-            subset->store_variable_d(WR_VAR(0, 7, 4), c.level.l1);
-        add(WR_VAR(0,  5, 15), &c);
-        add(WR_VAR(0,  6, 15), &c);
-        add(WR_VAR(0, 11, 61), &c);
-        add(WR_VAR(0, 11, 62), &c);
-        return true;
-    }
-
     virtual void to_subset(const Msg& msg, wreport::Subset& subset)
     {
         TempBase::to_subset(msg, subset);
@@ -285,6 +323,17 @@ struct TempWMO : public TempBase
             const msg::Context* c = *i;
             // Skip non-presure levels
             if (!c->find_vsig()) continue;
+            // Skip levels not for us
+            if (!(
+                  c->find(WR_VAR(0, 10,   8))
+               || c->find(WR_VAR(0, 12, 101))
+               || c->find(WR_VAR(0, 12, 103))
+               || c->find(WR_VAR(0, 11,   1))
+               || c->find(WR_VAR(0, 11,   2))) && (
+                  c->find(WR_VAR(0, 11, 61))
+               || c->find(WR_VAR(0, 11, 62)))
+               )
+                continue;
             do_D03054(*c);
             ++group_count;
         }
@@ -300,8 +349,13 @@ struct TempWMO : public TempBase
             // Iterate backwards to get pressure levels sorted from the higher
             // to the lower pressure
             const msg::Context* c = *i;
-            // Skip non-presure levels
+            // Skip non-pressure levels
             if (c->level.ltype1 != 100) continue;
+            // Skip levels not for us
+            if (!(
+                  c->find(WR_VAR(0, 11, 61))
+               || c->find(WR_VAR(0, 11, 62))))
+                continue;
             if (do_D03051(*c))
                 ++group_count;
         }
@@ -309,6 +363,97 @@ struct TempWMO : public TempBase
     }
 };
 
+struct TempRadar : public TempBase
+{
+    TempRadar(const Exporter::Options& opts, const Msgs& msgs)
+        : TempBase(opts, msgs) {}
+
+    virtual const char* name() const { return TEMP_RADAR_NAME; }
+    virtual const char* description() const { return TEMP_RADAR_DESC; }
+
+    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    {
+        TempBase::setupBulletin(bulletin);
+
+        bulletin.type = 6;
+        bulletin.subtype = 1;
+        bulletin.localsubtype = 255;
+
+        // Data descriptor section
+        bulletin.datadesc.clear();
+        bulletin.datadesc.push_back(WR_VAR(0,  1,  33));
+        bulletin.datadesc.push_back(WR_VAR(0,  1,  34));
+        bulletin.datadesc.push_back(WR_VAR(3,  1,   1));
+        bulletin.datadesc.push_back(WR_VAR(3,  1,  11));
+        bulletin.datadesc.push_back(WR_VAR(3,  1,  12));
+        bulletin.datadesc.push_back(WR_VAR(3,  1,  22));
+        bulletin.datadesc.push_back(WR_VAR(0,  2,   3));
+        bulletin.datadesc.push_back(WR_VAR(0,  2, 121));
+        bulletin.datadesc.push_back(WR_VAR(0,  2, 125));
+        bulletin.datadesc.push_back(WR_VAR(1,  7,   0));
+        bulletin.datadesc.push_back(WR_VAR(0, 31,   1));
+        bulletin.datadesc.push_back(WR_VAR(0,  7,   7));
+        bulletin.datadesc.push_back(WR_VAR(0, 11,   1));
+        bulletin.datadesc.push_back(WR_VAR(0, 11,   2));
+        bulletin.datadesc.push_back(WR_VAR(0, 33,   2));
+        bulletin.datadesc.push_back(WR_VAR(0, 11,   6));
+        bulletin.datadesc.push_back(WR_VAR(0, 33,   2));
+        bulletin.datadesc.push_back(WR_VAR(0, 11,  50));
+        bulletin.load_tables();
+    }
+
+    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    {
+        TempBase::to_subset(msg, subset);
+        add(WR_VAR(0, 1, 33), c_station);
+        add(WR_VAR(0, 1, 34), c_station);
+        do_D01001(); // station id
+        do_D01011(); // date
+        do_D01012(); // date
+        do_D01022(); // date
+        add(WR_VAR(0,  2,  3), c_gnd_instant, DBA_MSG_MEAS_EQUIP_TYPE);
+        add(WR_VAR(0, 2, 121), c_gnd_instant);
+        add(WR_VAR(0, 2, 125), c_gnd_instant);
+
+        // Undef for now, we fill it later
+        size_t rep_count_pos = subset.size();
+        subset.store_variable_undef(WR_VAR(0, 31, 1));
+
+        // Sounding levels
+        int group_count = 0;
+        for (std::vector<msg::Context*>::const_reverse_iterator i = msg.data.rbegin();
+                i != msg.data.rend(); ++i)
+        {
+            // Iterate backwards to get pressure levels sorted from the higher
+            // to the lower pressure
+            const msg::Context* c = *i;
+            // We are only interested in height levels
+            if (c->level.ltype1 != 102) continue;
+            if (const Var* var = c->find(WR_VAR(0, 7, 7)))
+                add(WR_VAR(0, 7, 7), var);
+            else
+                subset.store_variable_d(WR_VAR(0, 7, 7), c->level.l1/1000.0);
+            add(WR_VAR(0, 11,   1), c);
+            static const Varcode codes[] = { WR_VAR(0, 11,   2), WR_VAR(0, 11,   6) };
+            for (unsigned i = 0; i < 2; ++i)
+            {
+                Varcode code = codes[i];
+                if (const wreport::Var *var = c->find(code))
+                {
+                    add(code, var);
+                    add(code, var->enqa(WR_VAR(0, 33,  2)));
+                } else {
+                    subset.store_variable_undef(code);
+                    subset.store_variable_undef(WR_VAR(0, 33,   2));
+                }
+            }
+            add(WR_VAR(0, 11,  50), c);
+            ++group_count;
+        }
+        subset[rep_count_pos].seti(group_count);
+    }
+};
+
 struct TempEcmwfLand : public TempBase
 {
     TempEcmwfLand(const Exporter::Options& opts, const Msgs& msgs)
@@ -465,13 +610,161 @@ struct TempEcmwfShip : public TempBase
     }
 };
 
-struct Pilot : public TempBase
+struct PilotWMO : public TempBase
 {
-    Pilot(const Exporter::Options& opts, const Msgs& msgs)
+    bool pressure_levs;
+
+    PilotWMO(const Exporter::Options& opts, const Msgs& msgs)
         : TempBase(opts, msgs) {}
 
-    virtual const char* name() const { return PILOT_NAME; }
-    virtual const char* description() const { return PILOT_DESC; }
+    virtual const char* name() const { return PILOT_WMO_NAME; }
+    virtual const char* description() const { return PILOT_WMO_DESC; }
+
+    virtual void to_bulletin(wreport::Bulletin& bulletin)
+    {
+        // Scan contexts to see if we are encoding pressure or height levels
+        unsigned has_press = 0;
+        unsigned has_height = 0;
+
+        for (Msgs::const_iterator mi = msgs.begin(); mi != msgs.end(); ++mi)
+        {
+            const Msg& msg = **mi;
+            for (std::vector<msg::Context*>::const_iterator i = msg.data.begin();
+                    i != msg.data.end(); ++i)
+            {
+                switch ((*i)->level.ltype1)
+                {
+                    case 100: ++has_press; break;
+                    case 102: ++has_height; break;
+                    case 103: ++has_height; break;
+                }
+            }
+        }
+
+        pressure_levs = has_press >= has_height;
+
+        // Continue with normal operations
+        TempBase::to_bulletin(bulletin);
+    }
+
+    virtual void setupBulletin(wreport::Bulletin& bulletin)
+    {
+        TempBase::setupBulletin(bulletin);
+
+        bulletin.type = 2;
+        bulletin.subtype = 1;
+        bulletin.localsubtype = 255;
+
+        // Data descriptor section
+        bulletin.datadesc.clear();
+        if (pressure_levs)
+            bulletin.datadesc.push_back(WR_VAR(3, 9, 50)); // For pressure levels
+        else
+            bulletin.datadesc.push_back(WR_VAR(3, 9, 51)); // For height levels
+        bulletin.load_tables();
+    }
+    virtual void to_subset(const Msg& msg, wreport::Subset& subset)
+    {
+        TempBase::to_subset(msg, subset);
+
+        // Identification of launch site and instrumentation
+        add(WR_VAR(0,  1,  1), DBA_MSG_BLOCK);
+        add(WR_VAR(0,  1,  2), DBA_MSG_STATION);
+        add(WR_VAR(0,  1, 11), DBA_MSG_IDENT);
+        add(WR_VAR(0,  2, 11), DBA_MSG_SONDE_TYPE);
+        add(WR_VAR(0,  2, 14), DBA_MSG_SONDE_TRACKING);
+        add(WR_VAR(0,  2,  3), DBA_MSG_MEAS_EQUIP_TYPE);
+
+        // Date/time of launch
+        add(WR_VAR(0,  8, 21), 18); // Launch time
+        add(WR_VAR(0,  4,  1), DBA_MSG_YEAR);
+        add(WR_VAR(0,  4,  2), DBA_MSG_MONTH);
+        add(WR_VAR(0,  4,  3), DBA_MSG_DAY);
+        add(WR_VAR(0,  4,  4), DBA_MSG_HOUR);
+        add(WR_VAR(0,  4,  5), DBA_MSG_MINUTE);
+        add(WR_VAR(0,  4,  6), DBA_MSG_SECOND);
+
+        // Horizontal and vertical coordinates of launch site
+        add(WR_VAR(0,  5,  1), DBA_MSG_LATITUDE);
+        add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
+        add(WR_VAR(0,  7, 30), DBA_MSG_HEIGHT_STATION);
+        add(WR_VAR(0,  7, 31), DBA_MSG_HEIGHT_BARO);
+        add(WR_VAR(0,  7,  7), DBA_MSG_HEIGHT_RELEASE);
+        add(WR_VAR(0, 33, 24), DBA_MSG_STATION_HEIGHT_QUALITY);
+
+        // Keep track of where we stored
+        int rep_count_pos = subset.size();
+        subset.store_variable_undef(WR_VAR(0, 31,  2));
+
+        /* Iterate backwards as we need to add levels in decreasing pressure order */
+        int group_count = 0;
+        for (std::vector<msg::Context*>::const_reverse_iterator i = msg.data.rbegin();
+                i != msg.data.rend(); ++i)
+        {
+            const msg::Context& c = **i;
+            // We only want levels with a vertical sounding significance
+            const Var* vss = c.find_vsig();
+            if (vss == NULL) continue;
+            if (pressure_levs && c.level.ltype1 != 100) continue;
+            if (!pressure_levs && c.level.ltype1 != 102) continue;
+
+            add(WR_VAR(0,  4, 86), &c);
+            add(WR_VAR(0,  8, 42), &c);
+            if (pressure_levs)
+            {
+                // Wind data at pressure levels
+                if (c.level.l1 == MISSING_INT)
+                    subset.store_variable_undef(WR_VAR(0, 7, 4));
+                else
+                    subset.store_variable_d(WR_VAR(0, 7, 4), c.level.l1);
+            } else {
+                // Wind data at heights
+                if (c.level.l1 == MISSING_INT)
+                    subset.store_variable_undef(WR_VAR(0, 7, 9));
+                else
+                    subset.store_variable_d(WR_VAR(0, 7, 9), c.level.l1);
+            }
+            add(WR_VAR(0,  5, 15), &c);
+            add(WR_VAR(0,  6, 15), &c);
+            add(WR_VAR(0, 11, 1), &c);
+            add(WR_VAR(0, 11, 2), &c);
+            ++group_count;
+        }
+        subset[rep_count_pos].seti(group_count);
+
+        // Wind shear data
+        rep_count_pos = subset.size();
+        subset.store_variable_undef(WR_VAR(0, 31,  1));
+        group_count = 0;
+        for (std::vector<msg::Context*>::const_reverse_iterator i = msg.data.rbegin();
+                i != msg.data.rend(); ++i)
+        {
+            // Iterate backwards to get pressure levels sorted from the higher
+            // to the lower pressure
+            const msg::Context& c = **i;
+            // Skip levels that do not fit
+            if (pressure_levs && c.level.ltype1 != 100) continue;
+            if (!pressure_levs && c.level.ltype1 != 102) continue;
+            if (pressure_levs)
+            {
+                if (do_D03051(c))
+                    ++group_count;
+            } else {
+                if (do_D03053(c))
+                    ++group_count;
+            }
+        }
+        subset[rep_count_pos].seti(group_count);
+    }
+};
+
+struct PilotEcmwf : public TempBase
+{
+    PilotEcmwf(const Exporter::Options& opts, const Msgs& msgs)
+        : TempBase(opts, msgs) {}
+
+    virtual const char* name() const { return PILOT_ECMWF_NAME; }
+    virtual const char* description() const { return PILOT_ECMWF_DESC; }
 
     virtual void setupBulletin(wreport::Bulletin& bulletin)
     {
@@ -524,11 +817,11 @@ struct Pilot : public TempBase
         /* 10 */ add(WR_VAR(0,  6,  1), DBA_MSG_LONGITUDE);
         /* 11 */ add(WR_VAR(0,  7,  1), DBA_MSG_HEIGHT_STATION);
 
-        int var_levcount = subset.size();
+        int rep_count_pos = subset.size();
         subset.store_variable_undef(WR_VAR(0, 31,  1));
 
         /* Iterate backwards as we need to add levels in decreasing pressure order */
-        int lev_no = 0;
+        int group_count = 0;
         for (int i = msg.data.size() - 1; i >= 0; --i)
         {
             msg::Context& c = *msg.data[i];
@@ -573,10 +866,10 @@ struct Pilot : public TempBase
             else
                 subset.store_variable_undef(WR_VAR(0, 11, 2));
 
-            ++lev_no;
+            ++group_count;
         }
 
-        subset[var_levcount].seti(lev_no);
+        subset[rep_count_pos].seti(group_count);
 
         if (!is_crex)
         {
@@ -643,13 +936,29 @@ struct TempShipFactory : public virtual TemplateFactory
         return auto_ptr<Template>(new TempEcmwfShip(opts, msgs));
     }
 };
-struct TempFactory : public virtual TempEcmwfFactory, TempWMOFactory
+struct TempRadarFactory : public virtual TemplateFactory
+{
+    TempRadarFactory() { name = TEMP_RADAR_NAME; description = TEMP_RADAR_DESC; }
+
+    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
+    {
+        return auto_ptr<Template>(new TempRadar(opts, msgs));
+    }
+};
+struct TempFactory : public TempEcmwfFactory, public TempWMOFactory, public TempRadarFactory
 {
     TempFactory() { name = TEMP_NAME; description = TEMP_DESC; }
 
     std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
     {
         const Msg& msg = *msgs[0];
+
+        // Get the type of equipment used
+        if (const wreport::Var* var = msg.find(WR_VAR(0, 2, 3), Level(1), Trange::instant()))
+            // Is it a Radar?
+            if (var->enq(0) == 3)
+                return TempRadarFactory::make(opts, msgs);
+
         // ECMWF temps use normal replication which cannot do more than 256 levels
         if (msg.data.size() > 260)
             return TempWMOFactory::make(opts, msgs);
@@ -660,13 +969,38 @@ struct TempFactory : public virtual TempEcmwfFactory, TempWMOFactory
             return TempEcmwfFactory::make(opts, msgs);
     }
 };
-struct PilotFactory : public TemplateFactory
+struct PilotWMOFactory : public virtual TemplateFactory
+{
+    PilotWMOFactory() { name = PILOT_WMO_NAME; description = PILOT_WMO_DESC; }
+
+    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
+    {
+        return auto_ptr<Template>(new PilotWMO(opts, msgs));
+    }
+};
+struct PilotEcmwfFactory : public virtual TemplateFactory
+{
+    PilotEcmwfFactory() { name = PILOT_ECMWF_NAME; description = PILOT_ECMWF_DESC; }
+
+    std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
+    {
+        return auto_ptr<Template>(new PilotEcmwf(opts, msgs));
+    }
+};
+struct PilotFactory : public virtual PilotEcmwfFactory, PilotWMOFactory
 {
     PilotFactory() { name = PILOT_NAME; description = PILOT_DESC; }
 
     std::auto_ptr<Template> make(const Exporter::Options& opts, const Msgs& msgs) const
     {
-        return auto_ptr<Template>(new Pilot(opts, msgs));
+        const Msg& msg = *msgs[0];
+        const Var* var = msg.get_sonde_tracking_var();
+        // Try with another one in case the first was just unset
+        if (!var) var = msg.get_meas_equip_type_var();
+        if (var)
+            return PilotWMOFactory::make(opts, msgs);
+        else
+            return PilotEcmwfFactory::make(opts, msgs);
     }
 };
 
@@ -680,7 +1014,10 @@ static const TemplateFactory* tempwmo = NULL;
 static const TemplateFactory* tempecmwf = NULL;
 static const TemplateFactory* tempecmwfland = NULL;
 static const TemplateFactory* tempecmwfship = NULL;
+static const TemplateFactory* tempradar = NULL;
 static const TemplateFactory* pilot = NULL;
+static const TemplateFactory* pilotwmo = NULL;
+static const TemplateFactory* pilotecmwf = NULL;
 
     if (!temp) temp = new TempFactory;
     if (!tempship) tempship = new TempShipFactory;
@@ -688,7 +1025,10 @@ static const TemplateFactory* pilot = NULL;
     if (!tempecmwf) tempecmwf = new TempEcmwfFactory;
     if (!tempecmwfland) tempecmwfland = new TempEcmwfLandFactory;
     if (!tempecmwfship) tempecmwfship = new TempEcmwfShipFactory;
+    if (!tempradar) tempradar = new TempRadarFactory;
     if (!pilot) pilot = new PilotFactory;
+    if (!pilotwmo) pilotwmo = new PilotWMOFactory;
+    if (!pilotecmwf) pilotecmwf = new PilotEcmwfFactory;
 
     r.register_factory(temp);
     r.register_factory(tempship);
@@ -696,7 +1036,10 @@ static const TemplateFactory* pilot = NULL;
     r.register_factory(tempecmwf);
     r.register_factory(tempecmwfland);
     r.register_factory(tempecmwfship);
+    r.register_factory(tempradar);
     r.register_factory(pilot);
+    r.register_factory(pilotwmo);
+    r.register_factory(pilotecmwf);
 }
 
 }
