@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005--2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
+ * Copyright (C) 2005--2014  ARPA-SIM <urpsim@smr.arpa.emr.it>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,18 +18,17 @@
  */
 
 #include "config.h"
-#include "msgapi.h"
-
-#ifdef HAVE_DBALLE_DB
-#include "dbapi.h"
-#include <dballe/db/db.h>
-#endif
+#include "dballe/core/verbose.h"
+#include "dballe/simple/msgapi.h"
+#include "dballe/simple/dbapi.h"
+#include "dballe/db/db.h"
 
 #include <cstring>	// memset
 #include <limits.h>
 #include <float.h>
 #include "handles.h"
 #include "error.h"
+#include "trace.h"
 
 extern "C" {
 #include <f77.h>
@@ -114,7 +113,7 @@ static inline int fromfortran(int val)
  * This is a sample code for a data insert session with the simplified interface:
  * \code
  *   call idba_presentati(dba, "myDSN", "mariorossi", "CippoLippo")
- *   call idba_preparati(dba, handle, "reuse", "add", "add")
+ *   call idba_preparati(dba, handle, "read", "add", "write")
  *   call idba_scopa(handle, "")
  *   call idba_setr(handle, "lat", 30.)
  *   call idba_setr(handle, "lon", 10.)
@@ -185,7 +184,8 @@ static inline int fromfortran(int val)
 #define MAX_SIMPLE 50
 #define MAX_SESSION 10
 
-#ifdef HAVE_DBALLE_DB
+using namespace std;
+
 struct HSession : public fortran::HBase
 {
 	DB* db;
@@ -193,22 +193,22 @@ struct HSession : public fortran::HBase
 	void start()
 	{
 		fortran::HBase::start();
-		db = new dballe::DB;
+		db = 0;
 	}
 
 	void stop()
 	{
-		delete db;
+        if (db) delete db;
 		fortran::HBase::stop();
 	}
 };
 
 struct fortran::Handler<HSession, MAX_SESSION> hsess;
-#endif
 
 struct HSimple : public fortran::HBase
 {
 	fortran::API* api;
+    fortran::SessionTracer trace;
 
 	void start()
 	{
@@ -235,36 +235,21 @@ static int usage_refcount = 0;
 
 static void lib_init()
 {
-	if (usage_refcount > 0)
-		return;
+    if (usage_refcount > 0)
+        return;
 
-	fortran::error_init();
-#ifdef HAVE_DBALLE_DB
-	hsess.init("DB-All.e database sessions", "MAX_CALLBACKS");
-#endif
-	hsimp.init("DB-All.e work sessions", "MAX_SIMPLE");
+    dba_verbose_init();
+
+    fortran::trace_init();
+    fortran::error_init();
+    hsess.init("DB-All.e database sessions", "MAX_CALLBACKS");
+    hsimp.init("DB-All.e work sessions", "MAX_SIMPLE");
 
 	++usage_refcount;
 }
 
-#if 0
-static inline int double_is_missing(double d)
-{
-	switch (fpclassify(d))
-	{
-		case FP_ZERO:
-			return 0;
-		case FP_NORMAL:
-			return d == MISSING_DOUBLE;
-		default:
-			return 1;
-	}
-}
-#endif
-
 extern "C" {
 
-#ifdef HAVE_DBALLE_DB
 /**
  * Start working with a DBALLE database.
  *
@@ -297,13 +282,13 @@ F77_INTEGER_FUNCTION(idba_presentati)(
 	GENPTR_CHARACTER(user)
 	GENPTR_CHARACTER(password)
 	const char* chosen_dsn;
-	char s_dsn[50];
+	char s_dsn[256];
 	char s_user[20];
 	char s_password[20];
 
 	try {
 		/* Import input string parameters */
-		cnfImpn(dsn, dsn_length, 49, s_dsn); s_dsn[49] = 0;
+		cnfImpn(dsn, dsn_length, 255, s_dsn); s_dsn[255] = 0;
 		cnfImpn(user, user_length, 19, s_user); s_user[19] = 0;
 		cnfImpn(password, password_length, 19, s_password); s_password[19] = 0;
 
@@ -324,19 +309,25 @@ F77_INTEGER_FUNCTION(idba_presentati)(
 		} else
 			chosen_dsn = s_dsn;
 
-		/* If dsn looks like a url, treat it accordingly */
-		if (DB::is_url(chosen_dsn))
-			hs.db->connect_from_url(chosen_dsn);
-		else
-			hs.db->connect(chosen_dsn, s_user, s_password);
+        /* If dsn looks like a url, treat it accordingly */
+        if (DB::is_url(chosen_dsn))
+        {
+            IF_TRACING(fortran::log_presentati_url(*dbahandle, chosen_dsn));
+            hs.db = DB::connect_from_url(chosen_dsn).release();
+        }
+        else
+        {
+            IF_TRACING(fortran::log_presentati_dsn(*dbahandle, chosen_dsn, s_user, s_password));
+            hs.db = DB::connect(chosen_dsn, s_user, s_password).release();
+        }
 
 		/* Open the database session */
 		return fortran::success();
 	} catch (error& e) {
+        hsess.release(*dbahandle);
 		return fortran::error(e);
 	}
 }
-#endif
 
 /**
  * Stop working with a DBALLE database
@@ -348,10 +339,10 @@ F77_SUBROUTINE(idba_arrivederci)(INTEGER(dbahandle))
 {
 	GENPTR_INTEGER(dbahandle)
 
-	// try {
-#ifdef HAVE_DBALLE_DB
+    IF_TRACING(fortran::log_arrivederci(*dbahandle));
+
+    // try {
         hsess.release(*dbahandle);
-#endif
 
 		/*
 		dba_shutdown does not exist anymore, but I keep this code commented out
@@ -369,7 +360,6 @@ F77_SUBROUTINE(idba_arrivederci)(INTEGER(dbahandle))
 }
 
 
-#ifdef HAVE_DBALLE_DB
 /**
  * Starts a session with dballe.
  *
@@ -403,10 +393,6 @@ F77_SUBROUTINE(idba_arrivederci)(INTEGER(dbahandle))
  * @param qcflag
  *    Controls access to data attributes and can have these values:
  *    \li \c "read" attributes cannot be modified in any way.
- *    \li \c "add" attributes can can be added to the database, but existing
- *    attributes cannot be modified.  Deletion of attributes is disabled.  This is
- *    used to insert new attribute in the database while preserving the attributes
- *    that were already present in it.
  *    \li \c "write" attributes can freely be added, overwritten and deleted.
  * @return
  *   The error indication for the function.
@@ -444,7 +430,7 @@ F77_INTEGER_FUNCTION(idba_preparati)(
 		*handle = hsimp.request();
 		HSession& hs = hsess.get(*dbahandle);
 		HSimple& h = hsimp.get(*handle);
-
+        IF_TRACING(h.trace.log_preparati(*dbahandle, *handle, c_anaflag, c_dataflag, c_attrflag));
 		h.api = new fortran::DbAPI(*hs.db, c_anaflag, c_dataflag, c_attrflag);
 
 		return fortran::success();
@@ -453,10 +439,9 @@ F77_INTEGER_FUNCTION(idba_preparati)(
 		return fortran::error(e);
 	}
 }
-#endif
 
 /**
- * Access a file with wheter messages
+ * Access a file with weather messages
  *
  * @retval handle
  *   The session handle returned by the function
@@ -507,6 +492,8 @@ F77_INTEGER_FUNCTION(idba_messaggi)(
 		//HSession& hs = hsess.get(*dbahandle);
 		HSimple& h = hsimp.get(*handle);
 
+        IF_TRACING(h.trace.log_messaggi(*handle, c_filename, c_mode, c_type));
+
 		h.api = new fortran::MsgAPI(c_filename, c_mode, c_type);
 
 		return fortran::success();
@@ -527,7 +514,9 @@ F77_INTEGER_FUNCTION(idba_fatto)(INTEGER(handle))
 {
 	GENPTR_INTEGER(handle)
 
-	try {
+    try {
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_fatto());
 		hsimp.release(*handle);
 		return fortran::success();
 	} catch (error& e) {
@@ -557,11 +546,15 @@ F77_INTEGER_FUNCTION(idba_scopa)(INTEGER(handle), CHARACTER(repinfofile) TRAIL(r
 	cnfImpn(repinfofile, repinfofile_length,  PATH_MAX, fname); fname[PATH_MAX - 1] = 0;
 
 	try {
-		HSimple& h = hsimp.get(*handle);
-		if (fname[0] == 0)
-			h.api->scopa(0);
-		else
-			h.api->scopa(fname);
+        HSimple& h = hsimp.get(*handle);
+        if (fname[0] == 0)
+        {
+            IF_TRACING(h.trace.log_scopa());
+            h.api->scopa(0);
+        } else {
+            IF_TRACING(h.trace.log_scopa(fname));
+            h.api->scopa(fname);
+        }
 
 		return fortran::success();
 	} catch (error& e) {
@@ -814,10 +807,14 @@ F77_INTEGER_FUNCTION(idba_seti)(
 		if (*value == MISSING_INT)
 		{
 			TRACEMISSING("int");
+            IF_TRACING(h.trace.log_unset(parm));
 			h.api->unset(parm);
 		}
 		else
+        {
+            IF_TRACING(h.trace.log_set(parm, *value));
 			h.api->seti(parm, *value);
+        }
 		return fortran::success();
 	} catch (error& e) {
 		return fortran::error(e);
@@ -856,10 +853,14 @@ F77_INTEGER_FUNCTION(idba_setb)(
 		if (*value == MISSING_BYTE)
 		{
 			TRACEMISSING("byte");
+            IF_TRACING(h.trace.log_unset(parm));
 			h.api->unset(parm);
 		}
 		else
+        {
+            IF_TRACING(h.trace.log_set(parm, *value));
 			h.api->setb(parm, *value);
+        }
 		return fortran::success();
 	} catch (error& e) {
 		return fortran::error(e);
@@ -899,10 +900,14 @@ F77_INTEGER_FUNCTION(idba_setr)(
 		if (*value == MISSING_REAL)
 		{
 			TRACEMISSING("real");
+            IF_TRACING(h.trace.log_unset(parm));
 			h.api->unset(parm);
 		}
 		else
+        {
+            IF_TRACING(h.trace.log_set(parm, *value));
 			h.api->setr(parm, *value);
+        }
 		return fortran::success();
 	} catch (error& e) {
 		return fortran::error(e);
@@ -941,10 +946,14 @@ F77_INTEGER_FUNCTION(idba_setd)(
 		if (*value == MISSING_DOUBLE)
 		{
 			TRACEMISSING("double");
+            IF_TRACING(h.trace.log_unset(parm));
 			h.api->unset(parm);
 		}
 		else
+        {
+            IF_TRACING(h.trace.log_set(parm, *value));
 			h.api->setd(parm, *value);
+        }
 		return fortran::success();
 	} catch (error& e) {
 		return fortran::error(e);
@@ -986,10 +995,14 @@ F77_INTEGER_FUNCTION(idba_setc)(
 		if (val[0] == 0)
 		{
 			TRACEMISSING("char");
+            IF_TRACING(h.trace.log_unset(parm));
 			h.api->unset(parm);
 		}
 		else
+        {
+            IF_TRACING(h.trace.log_set(parm, val));
 			h.api->setc(parm, val);
+        }
 
 		return fortran::success();
 	} catch (error& e) {
@@ -1011,6 +1024,7 @@ F77_INTEGER_FUNCTION(idba_setcontextana)(
 	GENPTR_INTEGER(handle)
 	try {
 		HSimple& h = hsimp.get(*handle);
+		IF_TRACING(h.trace.log_func("setcontextana"));
 		h.api->setcontextana();
 
 		return fortran::success();
@@ -1085,6 +1099,9 @@ F77_INTEGER_FUNCTION(idba_setlevel)(
 	GENPTR_INTEGER(l2)
 	try {
 		HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_setlevel(
+                fromfortran(*ltype1), fromfortran(*l1),
+                fromfortran(*ltype2), fromfortran(*l2)));
 		h.api->setlevel(
 			fromfortran(*ltype1), fromfortran(*l1),
 			fromfortran(*ltype2), fromfortran(*l2));
@@ -1156,6 +1173,7 @@ F77_INTEGER_FUNCTION(idba_settimerange)(
 	GENPTR_INTEGER(p2)
 	try {
 		HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_settimerange(fromfortran(*ptype), fromfortran(*p1), fromfortran(*p2)));
 		h.api->settimerange(
 			fromfortran(*ptype), fromfortran(*p1), fromfortran(*p2));
 
@@ -1251,6 +1269,10 @@ F77_INTEGER_FUNCTION(idba_setdate)(
 	GENPTR_INTEGER(sec)
 	try {
 		HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_setdate(
+                fromfortran(*year), fromfortran(*month), fromfortran(*day),
+                fromfortran(*hour), fromfortran(*min), fromfortran(*sec)));
+
 		h.api->setdate(
 			fromfortran(*year), fromfortran(*month), fromfortran(*day),
 			fromfortran(*hour), fromfortran(*min), fromfortran(*sec));
@@ -1299,6 +1321,9 @@ F77_INTEGER_FUNCTION(idba_setdatemin)(
 	GENPTR_INTEGER(sec)
 	try {
 		HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_setdate(
+                fromfortran(*year), fromfortran(*month), fromfortran(*day),
+                fromfortran(*hour), fromfortran(*min), fromfortran(*sec), "min"));
 		h.api->setdatemin(
 			fromfortran(*year), fromfortran(*month), fromfortran(*day),
 			fromfortran(*hour), fromfortran(*min), fromfortran(*sec));
@@ -1347,6 +1372,9 @@ F77_INTEGER_FUNCTION(idba_setdatemax)(
 	GENPTR_INTEGER(sec)
 	try {
 		HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_setdate(
+                fromfortran(*year), fromfortran(*month), fromfortran(*day),
+                fromfortran(*hour), fromfortran(*min), fromfortran(*sec), "max"));
 		h.api->setdatemax(
 			fromfortran(*year), fromfortran(*month), fromfortran(*day),
 			fromfortran(*hour), fromfortran(*min), fromfortran(*sec));
@@ -1385,6 +1413,7 @@ F77_INTEGER_FUNCTION(idba_unset)(
 
 	try {
 		HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_unset(parm));
 		h.api->unset(parm);
 
 		return fortran::success();
@@ -1394,18 +1423,45 @@ F77_INTEGER_FUNCTION(idba_unset)(
 }
 
 /**
- * Remove all parameters from the input record
+ * Remove all B* parameters from the input record
  * 
  * @param handle
  *   Handle to a DBALLE session
  */
-F77_SUBROUTINE(idba_unsetall)(
-		INTEGER(handle))
+F77_INTEGER_FUNCTION(idba_unsetb)(
+        INTEGER(handle))
 {
-	GENPTR_INTEGER(handle)
+    GENPTR_INTEGER(handle)
 
-	HSimple& h = hsimp.get(*handle);
-	h.api->unsetall();
+    try {
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_func("unsetb"));
+        h.api->unsetb();
+        return fortran::success();
+    } catch (error& e) {
+        return fortran::error(e);
+    }
+}
+
+/**
+ * Remove all parameters from the input record
+ *
+ * @param handle
+ *   Handle to a DBALLE session
+ */
+F77_INTEGER_FUNCTION(idba_unsetall)(
+        INTEGER(handle))
+{
+    GENPTR_INTEGER(handle)
+
+    try {
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_func("unsetall"));
+        h.api->unsetall();
+        return fortran::success();
+    } catch (error& e) {
+        return fortran::error(e);
+    }
 }
 
 /**
@@ -1430,7 +1486,9 @@ F77_INTEGER_FUNCTION(idba_quantesono)(
 
 	try {
 		HSimple& h = hsimp.get(*handle);
+		IF_TRACING(h.trace.log_quantesono());
 		*count = h.api->quantesono();
+        IF_TRACING(fortran::log_result(*count));
 
 		return fortran::success();
 	} catch (error& e) {
@@ -1456,6 +1514,7 @@ F77_INTEGER_FUNCTION(idba_elencamele)(INTEGER(handle))
 
 	try {
 		HSimple& h = hsimp.get(*handle);
+		IF_TRACING(h.trace.log_func("elencamele"));
 		h.api->elencamele();
 
 		return fortran::success();
@@ -1485,7 +1544,9 @@ F77_INTEGER_FUNCTION(idba_voglioquesto)(
 
 	try {
 		HSimple& h = hsimp.get(*handle);
-		*count = h.api->voglioquesto();
+        IF_TRACING(h.trace.log_voglioquesto());
+        *count = h.api->voglioquesto();
+        IF_TRACING(fortran::log_result(*count));
 
 		return fortran::success();
 	} catch (error& e) {
@@ -1516,7 +1577,9 @@ F77_INTEGER_FUNCTION(idba_dammelo)(
 
 	try {
 		HSimple& h = hsimp.get(*handle);
-		const char* res = h.api->dammelo();
+        IF_TRACING(h.trace.log_dammelo());
+        const char* res = h.api->dammelo();
+        IF_TRACING(fortran::log_result(res));
 		if (!res)
 			cnfExprt("", parameter, parameter_length);
 		else
@@ -1551,8 +1614,9 @@ F77_INTEGER_FUNCTION(idba_prendilo)(
 {
 	GENPTR_INTEGER(handle)
 	try {
-		HSimple& h = hsimp.get(*handle);
-		h.api->prendilo();
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_func("prendilo"));
+        h.api->prendilo();
 
 		return fortran::success();
 	} catch (error& e) {
@@ -1575,8 +1639,9 @@ F77_INTEGER_FUNCTION(idba_dimenticami)(
 {
 	GENPTR_INTEGER(handle)
 	try {
-		HSimple& h = hsimp.get(*handle);
-		h.api->dimenticami();
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_func("dimenticami"));
+        h.api->dimenticami();
 
 		return fortran::success();
 	} catch (error& e) {
@@ -1600,7 +1665,9 @@ F77_INTEGER_FUNCTION(idba_voglioancora)(INTEGER(handle), INTEGER(count))
 	GENPTR_INTEGER(count)
 	try {
 		HSimple& h = hsimp.get(*handle);
-		*count = h.api->voglioancora();
+        IF_TRACING(h.trace.log_voglioancora());
+        *count = h.api->voglioancora();
+        IF_TRACING(fortran::log_result(*count));
 
 		return fortran::success();
 	} catch (error& e) {
@@ -1628,7 +1695,9 @@ F77_INTEGER_FUNCTION(idba_ancora)(
 
 	try {
 		HSimple& h = hsimp.get(*handle);
-		const char* res = h.api->ancora();
+        IF_TRACING(h.trace.log_ancora());
+        const char* res = h.api->ancora();
+        IF_TRACING(fortran::log_result(res));
 		if (!res)
 			cnfExprt("", parameter, parameter_length);
 		else
@@ -1673,6 +1742,7 @@ F77_INTEGER_FUNCTION(idba_critica)(
 
 	try {
 		HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_func("critica"));
 		h.api->critica();
 
 		return fortran::success();
@@ -1706,6 +1776,7 @@ F77_INTEGER_FUNCTION(idba_scusa)(INTEGER(handle))
 
 	try {
 		HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_func("scusa"));
 		h.api->scusa();
 
 		return fortran::success();
@@ -1714,6 +1785,118 @@ F77_INTEGER_FUNCTION(idba_scusa)(INTEGER(handle))
 	}
 }
 
+F77_INTEGER_FUNCTION(idba_messages_open_input)(
+        INTEGER(handle),
+        CHARACTER(filename),
+        CHARACTER(mode),
+        CHARACTER(format),
+        LOGICAL(simplified)
+        TRAIL(filename)
+        TRAIL(mode)
+        TRAIL(format))
+{
+    GENPTR_INTEGER(handle)
+    GENPTR_CHARACTER(filename)
+    GENPTR_CHARACTER(mode)
+    GENPTR_CHARACTER(format)
+    GENPTR_LOGICAL(simplified)
+    char c_filename[512];
+    char c_mode[10];
+    char c_format[10];
+    cnfImpn(filename, filename_length,  511, c_filename); c_filename[511] = 0;
+    cnfImpn(mode, mode_length,  9, c_mode); c_mode[9] = 0;
+    cnfImpn(format, format_length,  9, c_format); c_format[9] = 0;
+
+    try {
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_messages_open_input(c_filename, c_mode, c_format, *simplified));
+        h.api->messages_open_input(c_filename, c_mode, parse_encoding(c_format), *simplified);
+
+        return fortran::success();
+    } catch (error& e) {
+        return fortran::error(e);
+    }
+}
+
+F77_INTEGER_FUNCTION(idba_messages_open_output)(
+        INTEGER(handle),
+        CHARACTER(filename),
+        CHARACTER(mode),
+        CHARACTER(format)
+        TRAIL(filename)
+        TRAIL(mode)
+        TRAIL(format))
+{
+    GENPTR_INTEGER(handle)
+    GENPTR_CHARACTER(filename)
+    GENPTR_CHARACTER(mode)
+    GENPTR_CHARACTER(format)
+    char c_filename[512];
+    char c_mode[10];
+    char c_format[10];
+    cnfImpn(filename, filename_length,  511, c_filename); c_filename[511] = 0;
+    cnfImpn(mode, mode_length,  9, c_mode); c_mode[9] = 0;
+    cnfImpn(format, format_length,  9, c_format); c_format[9] = 0;
+
+    try {
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_messages_open_output(c_filename, c_mode, c_format));
+        h.api->messages_open_output(c_filename, c_mode, parse_encoding(c_format));
+
+        return fortran::success();
+    } catch (error& e) {
+        return fortran::error(e);
+    }
+}
+
+F77_INTEGER_FUNCTION(idba_messages_read_next)(INTEGER(handle), LOGICAL(found))
+{
+    GENPTR_INTEGER(handle)
+    GENPTR_LOGICAL(found)
+    try {
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_messages_read_next());
+        *found = h.api->messages_read_next();
+        IF_TRACING(fortran::log_result(*found));
+
+        return fortran::success();
+    } catch (error& e) {
+        return fortran::error(e);
+    }
+}
+
+F77_INTEGER_FUNCTION(idba_messages_write_next)(INTEGER(handle), CHARACTER(template_name) TRAIL(template_name))
+{
+    GENPTR_INTEGER(handle)
+    GENPTR_CHARACTER(template_name)
+    char c_template_name[64];
+    cnfImpn(template_name, template_name_length, 63, c_template_name); c_template_name[63] = 0;
+    try {
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_messages_write_next(c_template_name));
+        h.api->messages_write_next(c_template_name);
+
+        return fortran::success();
+    } catch (error& e) {
+        return fortran::error(e);
+    }
+}
+
+F77_INTEGER_FUNCTION(idba_remove_all)(
+        INTEGER(handle))
+{
+    GENPTR_INTEGER(handle)
+
+    try {
+        HSimple& h = hsimp.get(*handle);
+        IF_TRACING(h.trace.log_func("remove_all"));
+        h.api->remove_all();
+        return fortran::success();
+    } catch (error& e) {
+        return fortran::error(e);
+    }
+}
+
 F77_INTEGER_FUNCTION(idba_spiegal)(
 		INTEGER(handle),
 		INTEGER(ltype1),
@@ -1810,6 +1993,7 @@ F77_INTEGER_FUNCTION(idba_test_input_to_output)(
 	}
 }
 
+
 }
 
 /*@}*/
