@@ -19,7 +19,7 @@ class Parser(OptionParser):
         sys.exit(2)
 
 if __name__ == "__main__":
-    parser = Parser(usage="usage: %prog [options]",
+    parser = Parser(usage="usage: %prog [options] [key1=val] [key2=val]",
             version="%prog "+ VERSION,
             description="Navigate a DB-ALLe database")
     parser.add_option("--dsn", default=os.environ.get("DBA_DB", "test"), help="DSN, or URL-like database definition, to use for connecting to the DB-All.e database (default: %default, can be set in the environment as DBA_DB)"),
@@ -29,31 +29,23 @@ if __name__ == "__main__":
     (opts, args) = parser.parse_args()
 
     import dballe
-    import string
-    import re
     import wx
 
     from provami.Model import *
-    #from provami.ModelSingleQuery import Model
-    from provami.MapCanvas import MapCanvas
-    from provami.MapChoice import MapChoice
-    #from provami.MapChoiceSingleQuery import MapChoice
-    #from provami.DateCanvas import DateCanvas
 
     from provami.ProvamiArtProvider import ProvamiArtProvider
     from provami.Navigator import Navigator
 
-    app = wx.PySimpleApp()
-    db = dballe.DB()
+    app = wx.App(False)
     if dballe.DB.is_url(opts.dsn):
-        db.connect_from_url(opts.dsn)
+        db = dballe.DB.connect_from_url(opts.dsn)
     else:
-        db.connect(opts.dsn, opts.user, opts.password)
+        db = dballe.DB.connect(opts.dsn, opts.user, opts.password)
     model = Model(db)
 
 
     for q in args:
-        model.filter.setFromString(q)
+        model.filter.set_from_string(q)
 
     wx.ArtProvider.PushProvider(ProvamiArtProvider())
 
